/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5195 $
 * $Date: 2012-03-11 09:24:51 +0100 (So, 11. Mär 2012) $
 * $Author: posch $
 * 
 */

package de.unihalle.informatik.Alida.dataio.provider.swing;



import de.unihalle.informatik.Alida.annotations.ALDDataIOProvider;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException.ALDDataIOProviderExceptionType;

import javax.swing.*;

import java.lang.reflect.Field;
import java.util.*;

/**
 * Class for generic loading/saving of enumerations via GUI in Alida.
 * 
 * @author posch, moeller
 */
@ALDDataIOProvider

public class ALDEnumDataIOSwing implements ALDDataIOSwing {

	/**
	 * Interface method to announce class for which IO is provided for.
	 * 
	 * @return  Collection of classes provided.
	 */
	@Override
	public Collection<Class<?>> providedClasses() {
		LinkedList<Class<?>> classes = new LinkedList<Class<?>>();
		classes.add( Enum.class);
		return classes;
	}
	
	/** 
	 * Generate GUI element for generic reading of enumeration types.
	 */
	@Override
	public JComponent createGUIElement(Field field, Class<?> cl, Object obj) {

		Vector<Object> fields = new Vector<Object>();

		Object[]    consts = cl.getEnumConstants();
		for ( Object c : consts ) {
			fields.add( c);
		}

		JComboBox combo = new JComboBox( fields);

		// set default; if none provided, select first entry
		if ( obj != null )
			combo.setSelectedItem(obj);
		else
			combo.setSelectedIndex(0);

		return combo;
	}

	@Override
  public void setValue(Field field, Class<?> cl, JComponent guiElement,
      Object value) throws ALDDataIOProviderException {
		if (!(guiElement instanceof JComboBox))
			throw new ALDDataIOProviderException(
					ALDDataIOProviderExceptionType.INVALID_GUI_ELEMENT, 
					"ALDEnumDataIOSwing: readData received invalid GUI element!");
		((JComboBox)guiElement).setSelectedItem(value);
  }

	@Override
	public Object readData(Field field, Class<?> cl, JComponent guiElement)
		throws ALDDataIOProviderException {
			if (!(guiElement instanceof JComboBox))
				throw new ALDDataIOProviderException(
						ALDDataIOProviderExceptionType.INVALID_GUI_ELEMENT, 
						"ALDEnumDataIOSwing: readData received invalid GUI element!");
		return ((JComboBox)guiElement).getSelectedItem();
	}

	@Override
	public JComponent writeData(Object obj) {
		JTextField textfield = new JTextField(25);
		textfield.setText( obj.toString());
		textfield.setEditable( false);
		return textfield;
	}
}
