/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5195 $
 * $Date: 2012-03-11 09:24:51 +0100 (So, 11. Mär 2012) $
 * $Author: posch $
 * 
 */

package de.unihalle.informatik.Alida.dataio.provider.swing;



import de.unihalle.informatik.Alida.annotations.ALDDataIOProvider;
import de.unihalle.informatik.Alida.dataio.provider.helpers.ALDTableWindow;
import de.unihalle.informatik.Alida.exceptions.*;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException.ALDDataIOProviderExceptionType;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableModel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.*;

/**
 * Class for generic loading/saving of 1D-arrays from/to GUI in Alida.
 * 
 * @author moeller
 */
@ALDDataIOProvider

public class ALDNativeArray1DDataIOSwing implements ALDDataIOSwing {

	/**
	 * List of supported classes.
	 */
	private static LinkedList<Class<?>> classes = null;
	
	/**
	 * Default constructor.
	 */
	public ALDNativeArray1DDataIOSwing() {
		if (classes == null) {
			classes = new LinkedList<Class<?>>();
			ALDNativeArray1DDataIOSwing.classes.add( Boolean[].class);
			ALDNativeArray1DDataIOSwing.classes.add( Byte[].class);
			ALDNativeArray1DDataIOSwing.classes.add( Double[].class);
			ALDNativeArray1DDataIOSwing.classes.add( Float[].class);
			ALDNativeArray1DDataIOSwing.classes.add( Integer[].class);
			ALDNativeArray1DDataIOSwing.classes.add( Short[].class);
			ALDNativeArray1DDataIOSwing.classes.add( String[].class);
			ALDNativeArray1DDataIOSwing.classes.add( boolean[].class);
			ALDNativeArray1DDataIOSwing.classes.add( byte[].class);
			ALDNativeArray1DDataIOSwing.classes.add( double[].class);
			ALDNativeArray1DDataIOSwing.classes.add( float[].class);
			ALDNativeArray1DDataIOSwing.classes.add( int[].class);
			ALDNativeArray1DDataIOSwing.classes.add( short[].class);
		}
	}
	
	/**
	 * Interface method to announce class for which IO is provided for.
	 * 
	 * @return  Collection of classes provided.
	 */
	@SuppressWarnings("unchecked")
  @Override
	public Collection<Class<?>> providedClasses() {
		return (Collection<Class<?>>)classes.clone();
	}
	
	/** 
	 * Generic reading of 1D arrays.
	 */
	/* (non-Javadoc)
	 * @see de.unihalle.informatik.Alida.helpers.ALDDataIOSwing#createGUIElement(java.lang.Class, java.lang.Object)
	 */
	@Override
	public JComponent createGUIElement(Field field, Class<?> cl, Object obj) {
		return new ArrayConfigButton(field, cl, obj);
	}

	@Override
  public void setValue(Field field, Class<?> cl, JComponent guiElement,
      Object value) throws ALDDataIOProviderException {
		if (!(guiElement instanceof ArrayConfigButton))
			throw new ALDDataIOProviderException(
					ALDDataIOProviderExceptionType.INVALID_GUI_ELEMENT, 
					"NativeArray1DDataIO: setValue() received invalid GUI element!");
		if (!(classes.contains(value.getClass())))
			throw new ALDDataIOProviderException(
					ALDDataIOProviderExceptionType.INVALID_GUI_ELEMENT, 
					"NativeArray1DDataIO: setValue() received wrong object type!");
		((ArrayConfigButton)guiElement).setValue(field, cl, value);
  }

	@Override
	public Object readData(Field field, Class<?> cl, JComponent guiElement) 
		throws ALDDataIOProviderException {
		if (!(guiElement instanceof ArrayConfigButton))
			throw new ALDDataIOProviderException(
					ALDDataIOProviderExceptionType.INVALID_GUI_ELEMENT, 
					"NativeArray1DDataIO: readData() received invalid GUI element!");
		return ((ArrayConfigButton)guiElement).readData(field, cl);
	}

	@Override
	public JComponent writeData(Object obj) throws ALDDataIOProviderException {
		if (!(classes.contains(obj.getClass())))
			throw new ALDDataIOProviderException(
					ALDDataIOProviderExceptionType.OBJECT_TYPE_ERROR, 
					"NativeArray1DDataIO: object to write has wrong type!");
		// return a button to show a window with the elements
		return new ArrayShowButton(obj);
	}

	/**
	 * GUI element for configuring 1D arrays of native data types and wrapper.
	 * <p>
	 * This button has an array configuration window attached to it 
	 * where specific data is stored and accessable.
	 * 
	 * @author moeller
	 */
	private class ArrayConfigButton extends JButton {

		/**
		 * Array configuration window.
		 */
		private ArrayConfigWindow confWin;

		/**
		 * Constructor.
		 * 
		 * @param field	Field of collection.
		 * @param cl	Class of collection.
		 * @param obj	Default object.
		 */
		@SuppressWarnings("unused")
    public ArrayConfigButton(Field field, Class<?> cl, Object obj) {
			super("Configure Native Array...");
			this.confWin = new ArrayConfigWindow();
			this.setActionCommand("show");
			this.addActionListener(this.confWin);
		}
		
		/**
		 * Gets the data from the configuration window.
		 * 
		 * @param field	Field of collection.
		 * @param cl	Class of collection.
		 * @param obj	Default object.
		 * @return Current data.
		 */
		public void setValue(Field field, Class<?> cl, Object value) {
			this.confWin.setValue(field, cl, value);
		}

		/**
		 * Gets the data from the configuration window.
		 * 
		 * @param field	Field of collection.
		 * @param cl	Class of collection.
		 * @param obj	Default object.
		 * @return Current data.
		 */
		public Object readData(Field field, Class<?> cl) {
			return this.confWin.readData(field, cl);
		}
	}	
	
	/**
	 * GUI element for displaying 2D arrays.
	 * 
	 * @author moeller
	 */
	private class ArrayShowButton extends JButton 
		implements ActionListener{

		/**
		 * Data to be displayed.
		 */
		private Object data;

		/**
		 * Constructor.
		 */
		public ArrayShowButton(Object obj) {
			super("Show data...");
			this.setActionCommand("showButtonPressed");
			this.addActionListener(this);
			this.data = obj;
		}
		
		@Override
    public void actionPerformed(ActionEvent e) {
			String cmd = e.getActionCommand(); 
			if (cmd.equals("showButtonPressed")) {
				Class<?> cl = this.data.getClass();
				DefaultTableModel outTabModel = null;
				if (cl.equals(Boolean[].class)) {
					Boolean [] field = (Boolean[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 0) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
						for (int h=0;h<height;++h) {
							outTabModel.setValueAt(field[h], h, 1);
						}
				}
				if (cl.equals(Byte[].class)) {
					Byte [] field = (Byte[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 1) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
					for (int h=0;h<height;++h) {
							outTabModel.setValueAt(field[h], h, 0);
					}
				}
				if (cl.equals(Double[].class)) {
					Double [] field = (Double[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 1) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
						for (int h=0;h<height;++h) {
							outTabModel.setValueAt(field[h], h, 0);
						}
				}
				if (cl.equals(Float[].class)) {
					Float [] field = (Float[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 1) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
						for (int h=0;h<height;++h) {
							outTabModel.setValueAt(field[h], h, 0);
						}
				}
				if (cl.equals(Integer[].class)) {
					Integer [] field = (Integer[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 1) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
						for (int h=0;h<height;++h) {
							outTabModel.setValueAt(field[h], h, 0);
						}
				}
				if (cl.equals(Short[].class)) {
					Short [] field = (Short[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 1) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
					for (int h=0;h<height;++h) {
						outTabModel.setValueAt(field[h], h, 0);
					}
				}
				if (cl.equals(String[].class)) {
					String [] field = (String[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 1) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
					for (int h=0;h<height;++h) {
						outTabModel.setValueAt(field[h], h, 0);
					}
				}
				if (cl.equals(boolean[].class)) {
					boolean [] field = (boolean[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 1) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
					for (int h=0;h<height;++h) {
						outTabModel.setValueAt(new Boolean(field[h]), h, 0);
					}
				}
				if (cl.equals(byte[].class)) {
					byte [] field = (byte[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 1) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
					for (int h=0;h<height;++h) {
						outTabModel.setValueAt(new Byte(field[h]), h, 0);
					}
				}
				if (cl.equals(double[].class)) {
					double [] field = (double[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 1) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
					for (int h=0;h<height;++h) {
						outTabModel.setValueAt(new Double(field[h]), h, 0);
					}
				}
				if (cl.equals(float[].class)) {
					float [] field = (float[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 1) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
					for (int h=0;h<height;++h) {
						outTabModel.setValueAt(new Float(field[h]), h, 0);
					}					
				}
				if (cl.equals(int[].class)) {
					int [] field = (int[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 1) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
					for (int h=0;h<height;++h) {
						outTabModel.setValueAt(new Integer(field[h]), h, 0);
					}
				}
				if (cl.equals(short[].class)) {
					short [] field = (short[])this.data;
					int height = field.length;
					outTabModel = new DefaultTableModel(height, 1) {
						@Override
						public boolean isCellEditable(int row, int column) {
							return false;
						}};
					for (int h=0;h<height;++h) {
						outTabModel.setValueAt(new Short(field[h]), h, 0);
					}
				}
				new ALDTableWindow(outTabModel).openWindow();
			}
    }
	}	

	/**
	 * Graphical user interface window for displaying 
	 * objects of class {@link ALDTableModel}.
	 * 
	 * @author moeller
	 */
	private class ArrayConfigWindow extends JFrame implements ActionListener {

		/**
		 * Data table (swing tables are nicer than imageJ tables).
		 */
		private JTable dataTab;
		
		/**
		 * Reference object to the data.
		 */
		private DefaultTableModel dataTabModel;
		
		/**
		 * Table component in window.
		 */
		private JPanel tabPanel;
		
		/**
		 * Scrollpane for table panel;
		 */
		private JScrollPane tableScroller;
		
		/**
		 * For convenience: always open last directory for saving.
		 */
		private File lastDir;

		/**
		 * Default constructor.
		 */
		public ArrayConfigWindow() {
			this.dataTabModel= new DefaultTableModel(5, 1);

			// init current directory with user directory
			this.lastDir= new File(System.getProperty("user.dir"));

			this.setupTable();
		}
		
		@SuppressWarnings({ "unused" })
    public void setValue(Field field, Class<?> cl, Object value) {
			if (   cl.equals(Boolean[].class)
					|| cl.equals(Byte[].class)
					|| cl.equals(Double[].class)
					|| cl.equals(Float[].class)
					|| cl.equals(Integer[].class)
					|| cl.equals(Short[].class)
					|| cl.equals(String[].class)) {
				Object [] array  = (Object[])value;
				int rows = array.length;
				this.dataTabModel = new DefaultTableModel(rows,1);
				for (int r=0;r<rows;++r) {
					Object val = array[r];
					String entry = new String();
					if (val != null)
						entry = val.toString();
					this.dataTabModel.setValueAt(entry, r, 0);
				}
			}
			if (cl.equals(boolean[].class)) {
				boolean [] array = (boolean[])value;
				int rows = array.length;
				this.dataTabModel = new DefaultTableModel(rows,1);
				for (int r=0;r<rows;++r) {
					this.dataTabModel.setValueAt(Boolean.toString(array[r]), r, 0);
				}
			}
			if (cl.equals(byte[].class)) {
				byte [] array = (byte[])value;
				int rows = array.length;
				this.dataTabModel = new DefaultTableModel(rows,1);
				for (int r=0;r<rows;++r) {
					this.dataTabModel.setValueAt(Byte.toString(array[r]), r, 0);	
				}
			}
			if (cl.equals(double[].class)) {
				double [] array = (double[])value;
				int rows = array.length;
				this.dataTabModel = new DefaultTableModel(rows,1);
				for (int r=0;r<rows;++r) {
					this.dataTabModel.setValueAt(Double.toString(array[r]), r, 0);
				}
			}
			if (cl.equals(float[].class)) {
				float [] array = (float[])value;
				int rows = array.length;
				this.dataTabModel = new DefaultTableModel(rows,1);
				for (int r=0;r<rows;++r) {
					this.dataTabModel.setValueAt(Float.toString(array[r]), r, 0);
				}
			}
			if (cl.equals(int[].class)) {
				int [] array = (int[])value;
				int rows = array.length;
				this.dataTabModel = new DefaultTableModel(rows,1);
				for (int r=0;r<rows;++r) {
					this.dataTabModel.setValueAt(Integer.toString(array[r]), r, 0);
				}
			}
			if (cl.equals(short[].class)) {
				short [] array = (short[])value;
				int rows = array.length;
				this.dataTabModel = new DefaultTableModel(rows,1);
				for (int r=0;r<rows;++r) {
					this.dataTabModel.setValueAt(Short.toString(array[r]), r, 0);
				}
			}
			// update the view
			this.dataTab.setModel(this.dataTabModel);
			this.tabPanel.updateUI();
		}

		@SuppressWarnings({ "unchecked", "unused" })
    public Object readData(Field field, Class<?> cl) {
			Vector<Object> data = this.dataTabModel.getDataVector();
			int rows = data.size();
			Object entry;
			if (cl.equals(Boolean[].class)) {
				Boolean [] tmpfield  = new Boolean[rows];
				for (int r=0;r<rows;++r) {
						entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
						tmpfield[r] = (entry != null) ?
												Boolean.valueOf((String)(entry)) : null;
				}
				return tmpfield;
			}
			if (cl.equals(Byte[].class)) {
				Byte [] tmpfield  = new Byte[rows];
				for (int r=0;r<rows;++r) {
					entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
					tmpfield[r] = (entry != null) ?
							Byte.valueOf((String)(entry)) : null;
				}
				return tmpfield;
			}
			if (cl.equals(Double[].class)) {
				Double [] tmpfield  = new Double[rows];
				for (int r=0;r<rows;++r) {
					entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
					tmpfield[r] = (entry != null) ?
							Double.valueOf((String)(entry)) : null;
				}
				return tmpfield;
			}
			if (cl.equals(Float[].class)) {
				Float [] tmpfield  = new Float[rows];
				for (int r=0;r<rows;++r) {
					entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
					tmpfield[r] = (entry != null) ?
							Float.valueOf((String)(entry)) : null;
				}
				return tmpfield;		
			}
			if (cl.equals(Integer[].class)) {
				Integer [] tmpfield  = new Integer[rows];
				for (int r=0;r<rows;++r) {
					entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
					tmpfield[r] = (entry != null) ?
							Integer.valueOf((String)(entry)) : null;
				}
				return tmpfield;
			}
			if (cl.equals(Short[].class)) {
				Short [] tmpfield  = new Short[rows];
				for (int r=0;r<rows;++r) {
					entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
					tmpfield[r] = (entry != null) ?
							Short.valueOf((String)(entry)) : null;
				}
				return tmpfield;				
			}
			if (cl.equals(String[].class)) {
				String [] tmpfield  = new String[rows];
				for (int r=0;r<rows;++r) {
					entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
					tmpfield[r] = (entry != null) ? (String)entry : null;
				}
				return tmpfield;
			}
			if (cl.equals(boolean[].class)) {
				boolean [] tmpfield  = new boolean[rows];
				for (int r=0;r<rows;++r) {
					entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
					tmpfield[r] = (entry != null) ?
							Boolean.valueOf((String)(entry)).booleanValue() : false;
				}
				return tmpfield;
			}
			if (cl.equals(byte[].class)) {
				byte [] tmpfield  = new byte[rows];
				for (int r=0;r<rows;++r) {
						entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
						tmpfield[r] = (entry != null) ?
											Byte.valueOf((String)(entry)).byteValue() : 0;
				}
				return tmpfield;
			}
			if (cl.equals(double[].class)) {
				double [] tmpfield  = new double[rows];
				for (int r=0;r<rows;++r) {
					entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
					tmpfield[r] = (entry != null) ?
							Double.valueOf((String)(entry)).doubleValue() : 0.0;
				}
				return tmpfield;
			}
			if (cl.equals(float[].class)) {
				float [] tmpfield  = new float[rows];
				for (int r=0;r<rows;++r) {
					entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
					tmpfield[r] = (entry != null) ?
							Float.valueOf((String)(entry)).floatValue() : 0;
				}
				return tmpfield;
			}
			if (cl.equals(int[].class)) {
				int [] tmpfield  = new int[rows];
				for (int r=0;r<rows;++r) {
					entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
					tmpfield[r] = (entry != null) ?
							Integer.valueOf((String)(entry)).intValue() : 0;
				}
				return tmpfield;
			}
			if (cl.equals(short[].class)) {
				short [] tmpfield  = new short[rows];
				for (int r=0;r<rows;++r) {
					entry = ((Vector<Object>)data.elementAt(r)).elementAt(0);
					tmpfield[r] = (entry != null) ?
							Short.valueOf((String)(entry)).shortValue() : 0;
				}
				return tmpfield;
			}
			return null; 
		}
		
		/**
		 * Saves the contents of the table to a user-specified file.
		 * <p>
		 * The file format is TSV, i.e. tabulator-separated values.
		 * The default ending is '.txt'. The user can select the file
		 * name through a file open dialog which pops-up on call of the function.
		 */
		public void saveTable() {

			File file= null;
			
			JFrame dummy= new JFrame();
			JFileChooser getSaveFileDialog= new JFileChooser(); 
			getSaveFileDialog.setFileFilter(new DataTabFileFilter());
			getSaveFileDialog.setFileSelectionMode(JFileChooser.FILES_ONLY);
			getSaveFileDialog.setCurrentDirectory(this.lastDir); 
			getSaveFileDialog.setSelectedFile(new File("ResultTab.txt"));
			getSaveFileDialog.setApproveButtonText("Speichern");
			getSaveFileDialog.setDialogTitle("Select file name");
			int returnVal = getSaveFileDialog.showOpenDialog(dummy);
	        if (returnVal == JFileChooser.APPROVE_OPTION) {
	            file = getSaveFileDialog.getSelectedFile();
	            this.lastDir= getSaveFileDialog.getCurrentDirectory();
	        }
	        
	        // obviously we did not get any name, can't do anything...
			if (file== null)
				return;
			
			// ... otherwise write table to selected file
			try{
				if (file.exists()) {
					int ret= 
						JOptionPane.showConfirmDialog(null,"The file exists already, would "
						+ "you like to overwrite it?", "Warning",JOptionPane.YES_NO_OPTION);
					switch(ret)
					{
					case JOptionPane.NO_OPTION:
					case JOptionPane.CANCEL_OPTION:
						return;
					case JOptionPane.YES_OPTION:
						// nothing special to do, just proceed
					}
				}
				FileWriter ow= new FileWriter(file.getPath());
				StringBuffer [] tab= ALDTableWindow.tableToString(this.dataTabModel);
				for (int i=0;i<tab.length;++i)
					ow.write(tab[i].toString());
				ow.close();
			} catch (IOException e) {
				JOptionPane.showMessageDialog(null,"Error!!! " +
						"Could not open output file " + file.getPath() + "!");
			}
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
    @Override
	  public void actionPerformed(ActionEvent e) {
			
			String command = e.getActionCommand();
			
			// displays the window
			if (e.getActionCommand().equals("show")) {
				this.setVisible(true);
			}
			// deletes all data from the table and sets row and column counts to zero
			if (e.getActionCommand().equals("clear")) {
				for (int i=0;i<this.dataTabModel.getRowCount();++i)
					for (int j=0;j<this.dataTabModel.getColumnCount();++j)
						this.dataTabModel.setValueAt(null, i, j);
			}
			// closes the GUI window
			if (e.getActionCommand().equals("close")) {
				this.setVisible(false);
			}
			// saves the table data in TSV format
			if (e.getActionCommand().equals("save")) {
				this.saveTable();
			}		
			// add a column
//			if (command.equals("addCol")) {
//				this.dataTabModel.addColumn(null);
//			}
			// add a row
			if (command.equals("addRow")) {
				this.dataTabModel.addRow(new Vector<Object>());
			}
			// delete column
//			if (command.equals("delCol")) {
//				if (this.dataTabModel.getColumnCount() == 1)
//					return;
//				this.dataTabModel.setColumnCount(this.dataTabModel.getColumnCount()-1);
//			}
			// delete row
			if (command.equals("delRow")) {
				if (this.dataTabModel.getRowCount() == 1)
					return;
				this.dataTabModel.removeRow(this.dataTabModel.getRowCount()-1);
			}
		}
		
		/**
		 * Initializes the data table window.
		 */
		private void setupTable() {
			
			// instantiate result table and put into scroll pane
			this.dataTab= new JTable(this.dataTabModel);
			this.tableScroller= new JScrollPane(this.dataTab);
			this.tableScroller.setPreferredSize(new Dimension (400,300));
			this.dataTab.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

			// add a toolbar
//			JButton addCol= new JButton("Add Column");
//			addCol.setActionCommand("addCol");
//			addCol.addActionListener(this);
//			addCol.setSize(new Dimension(80,35));
			JButton addRow= new JButton("Add Row");
			addRow.setActionCommand("addRow");
			addRow.addActionListener(this);
			addRow.setSize(new Dimension(80,35));
//			JButton delCol= new JButton("Delete Column");
//			delCol.setActionCommand("delCol");
//			delCol.addActionListener(this);
//			delCol.setSize(new Dimension(80,35));
			JButton delRow= new JButton("Delete Row");
			delRow.setActionCommand("delRow");
			delRow.addActionListener(this);
			delRow.setSize(new Dimension(80,35));

			JButton tabSave= new JButton("Save");
			tabSave.setActionCommand("save");
			tabSave.addActionListener(this);
			tabSave.setSize(new Dimension(80,35));
			JButton tabClear= new JButton("Clear");
			tabClear.setActionCommand("clear");
			tabClear.addActionListener(this);
			tabClear.setSize(new Dimension(80,35));
			JButton tabClose= new JButton("Close");
			tabClose.setActionCommand("close");
			tabClose.addActionListener(this);
			tabClose.setSize(new Dimension(80,35));
			JPanel tabToolPanel= new JPanel();
			tabToolPanel.setLayout(new FlowLayout());
//			tabToolPanel.add(addCol);
			tabToolPanel.add(addRow);
//			tabToolPanel.add(delCol);
			tabToolPanel.add(delRow);
			tabToolPanel.add(tabSave);
			tabToolPanel.add(tabClear);
			tabToolPanel.add(tabClose);
			JToolBar tabTools= new JToolBar("");
			tabTools.add(tabToolPanel);
			
			this.tabPanel= new JPanel();
			this.tabPanel.setLayout(new BorderLayout());
			this.tabPanel.add(this.tableScroller, BorderLayout.CENTER);
			this.tabPanel.add(tabTools, BorderLayout.SOUTH);

			this.setSize(500, 250);
			this.setTitle("");
			this.add(this.tabPanel);
			
			// ... then show it!
			this.setVisible(false);
		}
		
		/**
		 * Internal class that realizes a FileFilter for text
		 * files where MiToBo table data is stored.
		 *	
		 * @author moeller
		 */
		protected class DataTabFileFilter extends FileFilter {
			
			/* (non-Javadoc)
			 * @see javax.swing.filechooser.FileFilter#accept(java.io.File)
			 */
			@Override
			public boolean accept(File f) {
				return (f.getName().endsWith(".txt") || f.isDirectory());
			}
			
			/* (non-Javadoc)
			 * @see javax.swing.filechooser.FileFilter#getDescription()
			 */
			@Override
			public String getDescription() {
				return "MiToBo Data Table Files (*.txt)";
			}
		}
	}
}
