/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5307 $
 * $Date: 2012-04-02 10:06:31 +0200 (Mo, 02. Apr 2012) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.Alida.dataio.provider.swing;



import de.unihalle.informatik.Alida.annotations.ALDDataIOProvider;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException.ALDDataIOProviderExceptionType;
import de.unihalle.informatik.Alida.helpers.ALDParser;

import javax.swing.*;

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.LinkedList;

/**
 * Class for loading/saving primitive datatypes in Alida.
 * 
 * @author moeller
 *
 */
@ALDDataIOProvider

public class ALDNativeDataIOSwing implements ALDDataIOSwing {

	/**
	 * Interface method to announce class for which IO is provided for.
	 * 
	 * @return  Collection of classes provided.
	 */
	@Override
	public Collection<Class<?>> providedClasses() {
		LinkedList<Class<?>> classes = new LinkedList<Class<?>>();
		classes.add( String.class);
		classes.add( boolean.class);
		classes.add( byte.class);
		classes.add( double.class);
		classes.add( float.class);
		classes.add( int.class);
		classes.add( long.class);
		classes.add( short.class);
		classes.add( Boolean.class);
		classes.add( Byte.class);
		classes.add( Double.class);
		classes.add( Float.class);
		classes.add( Integer.class);
		classes.add( Long.class);
		classes.add( Short.class);
		return classes;
	}
	
	@Override
	public JComponent createGUIElement(Field field, Class<?> cl, Object obj) {
		if ( cl != boolean.class && cl != Boolean.class ) {
			JTextField textfield = new JTextField(25);
			if ( obj != null )
				if ( supported1DArray( cl) ) {
					textfield.setText( ALDParser.arrayToString( obj));
				} else {
					textfield.setText( obj.toString());
				}
			return textfield;
		}
		JCheckBox checkbox = new JCheckBox();
		if ( obj != null )
			checkbox.setSelected( ((Boolean)obj).booleanValue() );
		return checkbox;
	}

	@Override
	public void setValue(Field field, Class<?> cl, JComponent guiElement,
			Object value) throws ALDDataIOProviderException {
		if ( cl != boolean.class && cl != Boolean.class ) {
			if (!(guiElement instanceof JTextField))
				throw new ALDDataIOProviderException(
						ALDDataIOProviderExceptionType.INVALID_GUI_ELEMENT, 
						"NativeDataIO: readData received invalid GUI element!");

			// 2D-arrays
			if (cl.getName().startsWith("[[")) {
				throw new ALDDataIOProviderException(
						ALDDataIOProviderExceptionType.UNSPECIFIED_ERROR, 
						"ALDNativeDataIO: 2D array reading not available!");
			}
			// 1D-arrays
			//  			else if ( supported1DArray( cl) ) {
			//  				((JTextField)guiElement).setText(ALDParser.arrayToString(value));
			//  			}
			else {
				((JTextField)guiElement).setText(new String(value.toString()));
			}
		}
		else {
			((JCheckBox)guiElement).setSelected(((Boolean)value).booleanValue());
		}
	}

	@Override
	public Object readData(Field field, Class<?> cl, JComponent guiElement) 
	throws ALDDataIOProviderException {

		if ( cl != boolean.class && cl != Boolean.class ) {

			if (!(guiElement instanceof JTextField))
				throw new ALDDataIOProviderException(
						ALDDataIOProviderExceptionType.INVALID_GUI_ELEMENT, 
						"NativeDataIO: readData received invalid GUI element!");

			String text = ((JTextField)guiElement).getText();
			// if there are no contents in the text field, indicate a problem...
			if (text.isEmpty())
				return null;

			if (cl.equals(String.class)) 
				return text;

			try {
				// native types
				if (cl.equals(boolean.class))
					return Boolean.valueOf(text);
				if (cl.equals(byte.class))
					return Byte.valueOf(text);
				if (cl.equals(double.class))
					return Double.valueOf(text);
				if (cl.equals(float.class))
					return Float.valueOf(text);
				if (cl.equals(int.class))
					return Integer.valueOf(text);
				if (cl.equals(long.class))
					return Long.valueOf(text);
				if (cl.equals(short.class))
					return Short.valueOf(text);
				// wrapper
				if (cl.equals(Boolean.class)) 
					return Boolean.valueOf(text);
				if (cl.equals(Byte.class))
					return Byte.valueOf(text);
				if (cl.equals(Double.class))
					return Double.valueOf(text);
				if (cl.equals(Float.class))
					return Float.valueOf(text);
				if (cl.equals(Integer.class))
					return Integer.valueOf(text);
				if (cl.equals(Long.class))
					return Long.valueOf(text);
				if (cl.equals(Short.class))
					return Short.valueOf(text);
				// 2D-arrays
				if (cl.getName().startsWith("[[")) {
					throw new ALDDataIOProviderException(
							ALDDataIOProviderExceptionType.UNSPECIFIED_ERROR, 
							"ALDNativeDataIO: 2D array reading not available!");
				}
				//    			// 1D-arrays
				//    			else if ( supported1DArray( cl) ) {
				//    				return ALDParser.readArray1D(cl, text);
				//    			}

			} catch (Exception e) {
				throw new ALDDataIOProviderException(
						ALDDataIOProviderExceptionType.UNSPECIFIED_ERROR, 
						"ALDNativeDataIO: something went wrong in parsing..." +
						e.getMessage());
			}
		} else {
			if (!(guiElement instanceof JCheckBox))
				throw new ALDDataIOProviderException(
						ALDDataIOProviderExceptionType.INVALID_GUI_ELEMENT, 
						"NativeDataIO: readData received invalid GUI element!");
			return new Boolean(((JCheckBox)guiElement).isSelected());
		}
		// we should never end up here
		throw new ALDDataIOProviderException(
				ALDDataIOProviderExceptionType.UNSPECIFIED_ERROR,
				"ALDNativeDataIOSwing - something went wrong, I don't know what...");
	}

	@Override
	public JComponent writeData(Object obj) {
		if ( obj.getClass() != boolean.class && obj.getClass() != Boolean.class ) {
			JTextField textfield = new JTextField(25);

			if ( supported1DArray( obj.getClass()) ) 
				textfield.setText( ALDParser.arrayToString( obj));
			else
				textfield.setText( obj.toString());

			textfield.setEditable( false);
			return textfield;
		}
		JCheckBox checkbox = new JCheckBox();
		if ( obj != null )
			checkbox.setSelected( (Boolean)obj);
		checkbox.setEnabled( false);
		return checkbox;

	}

	/**
	 * Checks if a given class is supported or not.
	 * 
	 * @param cl	Class to check.
	 * @return	True if class is supported.
	 */
	private boolean supported1DArray( Class cl ) {
		if ( cl == Boolean[].class ||
				cl == Byte[].class ||
				cl == Double[].class ||
				cl == Float[].class ||
				cl == Integer[].class ||
				cl == Short[].class ||
				cl == String[].class ||
				cl == boolean[].class ||
				cl == byte[].class ||
				cl == double[].class ||
				cl == float[].class ||
				cl == int[].class ||
				cl == short[].class   )
			return true;
		return false;
	}
}
