/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5195 $
 * $Date: 2012-03-11 09:24:51 +0100 (So, 11. Mär 2012) $
 * $Author: posch $
 * 
 */

package de.unihalle.informatik.Alida.demo;

import de.unihalle.informatik.Alida.annotations.*;


/**
 * This class is a toy implementation of experimental data for
 * a demo of Alida's parametrized classes.
 * The complete experiment consists of a number of indepenent repetitions of
 * sub experiments. In each of these the same features (measurments) are recorded.
 * 
 * The measurements a represented in 
 * a 2D array of Doubles, where each column represents
 * one sub experiment and the rows the distinct features.
 * 
 * @author posch
 *
 */
@ALDParametrizedClass

public class ExperimentalData {
    @ALDClassParameter(label="description")
    private String description = null;

    @ALDClassParameter(label="data")
    private Double[][] data = null;

    /** are the data normalized
      */
    @ALDClassParameter(label="Is normalized")
    private boolean normalized = false;

    /** Standard constructor is needed
      */
    public ExperimentalData() {
    }

    /** Constructor for an experiment.
      * Normalized is assumed to be false.
      *
      * @param  description   a textual desciption of the experiment
      * @param  data   measurements
      */
    public ExperimentalData( String description, Double[][] data) {    
        this( description, data, false);
    }

    /** Constructor for an experiment.
      *
      * @param  description   a textual desciption of the experiment
      * @param  data   measurements
      * @param  normalized   are the data normalized
      */
    public ExperimentalData( String description, Double[][] data, boolean normalized) {    
        this.normalized = normalized;
        this.description = description;
        this.setData( data, normalized);
    }

    /** Return number of sub experiments
      * @return number of sub experiments
    */
    public int getNumExperiments() {
        return data[0].length;
    }

    /** Return number of features
      * @return number of features
    */
    public int getNumFeatures() {
        return data.length;
    }

    /** Return the data
     * @return data matrix
     */
    public Double[][] getData() {
        return this.data;
    }

    /** Set the data, assumed to be not normalized.
     * @param data matrix
     */
    private void setData( Double[][] data) {
        setData( data, false);
    }

    /** Set the data
     * @param data matrix
     * @param normalized are the data normalized?
     */
    private void setData( Double[][] data, boolean normalized) {
        this.data = data;
        this.normalized = normalized;

    }

    /** Get description.
      * @return description
      */
    public String getDescription(){
        return description;
    }

    /** Set description.
      * @param description
      */
    private void setDescription( String description){
        this.description = description;
    }
}
