/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5413 $
 * $Date: 2012-04-12 11:01:03 +0200 (Do, 12. Apr 2012) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.Alida.helpers;



import de.unihalle.informatik.Alida.annotations.ALDDerivedClass;
import de.unihalle.informatik.Alida.annotations.indexing.SezPozAdapter;

import java.util.Collection;
import java.util.LinkedList;

import net.java.sezpoz.Index;
import net.java.sezpoz.IndexItem;

/**
 * Supplies helper methods to, e.g., lookup classes with annotations.
 * 
 * @author posch
 */

public class ALDClassInfo {

	/**
	 * Debug flag for internal usage only.
	 */
	private static boolean debug = false;

	/**
	 * Collects all derived class of given class.
	 * <p>
	 * The method searches for all derived classes of the given class
	 * among all classes annotated with @ALDDerivedClass that are found
	 * in the classpath. In this context derived classes are classes which
	 * either implement the specified interface or extend the given class. 
	 *
	 * @param	cl	Class for which derived classes are requested.
	 * @return	List of extending classes including class itself.
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public static Collection<Class> lookupExtendingClasses( Class cl) {
		// init result list
		LinkedList<Class> extendingClasses = new LinkedList<Class>();

		if ( debug ) 
			System.out.println("ALDDataIOManager::lookupExtendingClasses class "+cl);
		
		// run through all classes annotated as Alida derived class
		Index<ALDDerivedClass,Object> indexItems =
				SezPozAdapter.load(ALDDerivedClass.class, Object.class);
		for( final IndexItem<ALDDerivedClass,Object> item : indexItems) {

			try {
				if ( cl.isAssignableFrom( Class.forName( item.className())) ) {
					extendingClasses.add( Class.forName( item.className()));
					if ( debug )
						System.out.println("ALDDataIOManager::lookupExtendingClasses found " + 
								item.className());
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return extendingClasses;
	}
}

