/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5462 $
 * $Date: 2012-04-17 17:07:15 +0200 (Di, 17. Apr 2012) $
 * $Author: misiak $
 * 
 */

package de.unihalle.informatik.Alida.operator.gui;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.util.*;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;

import net.java.sezpoz.Index;
import net.java.sezpoz.IndexItem;

import de.unihalle.informatik.Alida.operator.*;


import de.unihalle.informatik.Alida.annotations.*;
import de.unihalle.informatik.Alida.helpers.ALDEnvironmentConfig;
import de.unihalle.informatik.Alida.annotations.ALDAOperator.Level;
import de.unihalle.informatik.Alida.annotations.indexing.SezPozAdapter;

/**
 * Main window for selecting Alida annotated operators for running.
 * 
 * @author Stefan Posch
 * @author Birgit Moeller
 */

public class ALDChooseOpNameFrame extends JFrame 
	implements TreeSelectionListener, ActionListener {

	/**
	 * Debug flag (not accessible from outside).
	 */
	private boolean debug = false;

	/**
	 * Name of selected operator.
	 */
	protected String opName = null;
	
	/**
	 * Mode of usage.
	 * <p>
	 * In application mode only operators annotated as applications
	 * are available, otherwise all annotated operators are displayed.
	 */
	protected ALDAOperator.Level level = ALDAOperator.Level.APPLICATION;
	
	/**
	 * Main panel of chooser window.
	 */
	protected JRootPane mainPanel;
	
	/**
	 * List of favorite operators to be unfolded at startup.
	 */
	Vector<String> favoriteOperators;
	
	/**
	 * Label to display selected operator.
	 */
	protected JLabel opNameJText;
	
	/**
	 * Tree of available operators.
	 */
	protected JTree opTree;
	
	/**
	 * Scroll pane displaying the operator tree.
	 */
	protected JScrollPane opTreePane;

	/**
	 * Icon to be shown in about box.
	 */
	protected ImageIcon aboutIcon;
	
	/**
	 * Title of frame.
	 */
	protected String titleString = 
		"Alida - OpRunner: simply choose an operator...";
	
	/**
	 * Constructor.
	 */
	public ALDChooseOpNameFrame() {
		// global language settings
		Locale.setDefault(Locale.ENGLISH);
		JComponent.setDefaultLocale(Locale.ENGLISH);

		String iconDataName = "/share/logo/Alida_logo.png";
		Image img = null;
		BufferedImage bi = null;
		Graphics g = null;
		InputStream is = null;
		try {
			ImageIcon icon;
			File iconDataFile = new File("./" + iconDataName);
			if(iconDataFile.exists()) {
				icon = new ImageIcon("./" + iconDataName);
				img = icon.getImage();
			}
			// try to find it inside a jar archive....
			else {
				is = ALDChooseOpNameFrame.class.getResourceAsStream(iconDataName);
				if (is == null) {
					System.err.println("Warning - cannot find icons...");
					img = new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
				}
				else {
					img = ImageIO.read(is);
				}
				bi= new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
				g = bi.createGraphics();
				g.drawImage(img, 0, 0, 20, 20, null);
			}
		} catch (IOException ex) {
			System.err.println("ALDChooseOpNameFrame - problems loading icons...!");
			img = new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
			bi= new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
			g = bi.createGraphics();
			g.drawImage(img, 0, 0, 20, 20, null);
		}
		this.aboutIcon = new ImageIcon(img);
		
		// read configuration
		this.favoriteOperators = initFavoriteOperators();

		String levelString = 
			ALDEnvironmentConfig.getConfigValue( "OPRUNNER", "LEVEL");
		if ( levelString != null ) {
			if ( levelString.equalsIgnoreCase( "application") )
				this.level = ALDAOperator.Level.APPLICATION;
			else
				this.level = ALDAOperator.Level.STANDARD;
		}

		// set up main window
		this.mainPanel = new JRootPane();
		BoxLayout ylayout = new BoxLayout( this.mainPanel, BoxLayout.Y_AXIS);
		this.mainPanel.setLayout( ylayout);

		// initialize the operator tree
		this.buildUpOpNameTree( this.level);
		
		// add tree view to main panel
		this.opTreePane = new JScrollPane( this.opTree);
		this.mainPanel.add(this.opTreePane,0);
		
		// do some global settings
		this.setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

		// TODO Does this anything? -> if not, remove...
		this.mainPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

		// add panel for displaying selected operator
		JLabel lab =  new JLabel("Selected Operator: ");
		this.opNameJText = new JLabel( "- none -");
		JPanel selelectePanel = new JPanel(); 
		selelectePanel.add( lab);
		selelectePanel.add(Box.createRigidArea(new Dimension(10, 0)));
		selelectePanel.add( this.opNameJText);

		// add to main panel
		this.mainPanel.add( selelectePanel);

		JButton configButton = new JButton("Configure Operator...");
		configButton.setActionCommand( "config");
		configButton.addActionListener(this);
		configButton.setBounds(50, 60, 80, 30);
		configButton.setAlignmentX(Component.LEFT_ALIGNMENT);

		JButton quitButton = new JButton("Quit");
		quitButton.setActionCommand( "quit");
		quitButton.addActionListener(this);
		quitButton.setBounds(50, 60, 80, 30);
		quitButton.setAlignmentX(Component.LEFT_ALIGNMENT);

		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.LINE_AXIS));
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(configButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(10, 0)));
		buttonPanel.add(quitButton);

		// add buttons to main panel
		this.mainPanel.add(buttonPanel);

		// add a nice menubar
		JMenuBar mainWindowMenu = new JMenuBar();
		JMenu fileM = new JMenu("File");
		JMenuItem quitItem = new JMenuItem("Quit");
		quitItem.setActionCommand("quit");
		quitItem.addActionListener(this);
		fileM.add(quitItem);
		JMenu opSelectM = new JMenu("Operator Level");
		ButtonGroup operatorLevelGroup = new ButtonGroup();
		// default
		JRadioButtonMenuItem radioItemApplication =
			new JRadioButtonMenuItem("Application");
		radioItemApplication.setActionCommand("viewApps");
		radioItemApplication.addActionListener(this);
		JRadioButtonMenuItem radioItemStandard = 
			new JRadioButtonMenuItem("Standard");
		radioItemStandard.setActionCommand("viewStd");
		radioItemStandard.addActionListener(this);
		operatorLevelGroup.add(radioItemApplication);
		operatorLevelGroup.add(radioItemStandard);
		opSelectM.add(radioItemApplication);
		opSelectM.add(radioItemStandard);

		if ( this.level == Level.APPLICATION)
			radioItemApplication.setSelected(true);
		else
			radioItemStandard.setSelected(true);

		JMenu helpM = this.generateHelpMenu();
		mainWindowMenu.add(fileM);
		mainWindowMenu.add(opSelectM);
		mainWindowMenu.add(Box.createHorizontalGlue());
		mainWindowMenu.add(helpM);

		// ... and go setup the frame
		this.setTitle(this.titleString);
		this.getContentPane().setPreferredSize(new Dimension(500, 700));
		this.pack();
		this.setJMenuBar(mainWindowMenu);
		this.add(this.mainPanel);
		this.setDefaultCloseOperation( DISPOSE_ON_CLOSE);
		this.setVisible(true);
	}
	
	/**
	 * Sets up the help menu.
	 * 
	 * @return Generated help menu.
	 */
	protected JMenu generateHelpMenu() {
		JMenu helpM = new JMenu("Help");
		JMenuItem itemHelp = new JMenuItem("Online Help");
		JMenuItem itemAbout = new JMenuItem("About Alida");
		itemHelp.addActionListener(
				OnlineHelpDisplayer.getHelpActionListener(itemHelp,"welcome"));
		itemAbout.setActionCommand("showAbout");
		itemAbout.addActionListener(this);
		helpM.add(itemHelp);
		helpM.add(itemAbout);
		return helpM;
	}

	/**
	 * Builds up the tree view of available operators.
	 * <p>
	 * Only operators annotated to be generically executed in the GUI
	 * and annotated according to the current mode of expertise are added.
	 */
	protected void buildUpOpNameTree(ALDAOperator.Level lev) {
		Vector<String> allOpNames = new Vector<String>();
		Node treeOfOpNames = new Node();

		if ( this.debug ) 
			System.out.println( "Looking up all annotated @ALDAOperators");

		Index<ALDAOperator, ALDOperator> indexItems = 
				SezPozAdapter.load(ALDAOperator.class, ALDOperator.class);
		for ( final IndexItem<ALDAOperator,ALDOperator> item : indexItems ) {

			if (   (      item.annotation().genericExecutionMode()
							   == ALDAOperator.ExecutionMode.ALL 
							||    item.annotation().genericExecutionMode() 
							   == ALDAOperator.ExecutionMode.SWING)
			    && (lev != Level.APPLICATION || item.annotation().level() == lev )) {
				String className = item.className();
				allOpNames.add( className);

				// build a tree of opNames according to packages
				String[] parts = className.split( "\\.");

				Node currentNode = treeOfOpNames;
				// loop over the part but the last one
				int i;
				for ( i = 0 ; i < parts.length-1 ; i++ ) {
					String part = parts[i];
					if ( currentNode.children.get( part) == null ) {
						Node newChild = new Node( (currentNode.fullName.equals("") ? 
																		"" : currentNode.fullName +".") + part);
						currentNode.children.put( part, newChild);
					}
					currentNode = currentNode.children.get( part);
				}
				currentNode.opNames.add( parts[i]);
			}
		}
		this.opTree = new JTree(treeOfOpNames.createTree( "ALDOperators"));
		this.opTree.addTreeSelectionListener(this);

		MouseListener ml = new MyMouseAdapter(this);
		this.opTree.addMouseListener(ml);

		hideLeaves( this.opTree, new TreePath( this.opTree.getModel().getRoot()));
	}


	/**
	 *  Hide all leaves of this tree below the given path.
	 *
	 * @param tree 			Tree in which to hide leaves.
	 * @param treePath 	Path below which leaves are hidden.
	 */
	private void hideLeaves( JTree tree, TreePath treePath) {
		OpTreeNode lastNode = (OpTreeNode)treePath.getLastPathComponent();
		TreeModel treeModel=tree.getModel();
		if ( ! lastNode.isLeaf() )
			tree.collapsePath(treePath);
		if ( this.favoriteOperators.contains( lastNode.getFullName()) )
			tree.makeVisible(treePath);

		if ( treeModel.getChildCount(lastNode) >= 0 ) {
			for ( int i = 0; i < treeModel.getChildCount(lastNode); i++  ) {
				TreeNode child = ( TreeNode )treeModel.getChild(lastNode, i);
				TreePath extendedPath = treePath.pathByAddingChild( child );
				hideLeaves( tree, extendedPath);
			}
		}      
	}

	/** 
	 * Initialize list of favorite operators to be unfolded at startup.
	 * <p>
	 * The list is initialized from environment configuration.
	 *
	 * @return List of favorite operators.
	 */
	private Vector<String> initFavoriteOperators() {
		String favoriteOpsConfigFile = 
			ALDEnvironmentConfig.getConfigValue( "OPRUNNER", "FAVORITEOPS");

		Vector<String> favoriteOps = new Vector<String>();
		if ( this.debug ) {
			System.out.println( 
					"ALDChooseOpNameFrame::initFavoriteOperators configfile = " + 
					favoriteOpsConfigFile);
		}

		if ( favoriteOpsConfigFile != null ) {
			try {
				BufferedReader reader = 
					new BufferedReader( new FileReader(favoriteOpsConfigFile));
				String line;
				while ( (line = reader.readLine()) != null ) {
					favoriteOps.add( line);
				}

			} catch (Exception e) {
				// nothing to be done
			}
		}
		return favoriteOps;
	}


	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {

		// local variables
		String command = e.getActionCommand();

		// configure button has been pressed
		if ( command.equals("config") ) {
			// do we have an operator name?
			if (this.opName != null ) {
				ALDOperatorControlFrame controlFrame = 
					new ALDOperatorControlFrame(this.opName);
				controlFrame.setVisible(true);

				//				// check if operator is interruptable
//				this.selectedOperator = this.chooser.getOpName();
//				Object operator;
//				try {
//					operator = Class.forName(this.selectedOperator).newInstance();
//				} catch (Exception e1) {
//					System.err.println("Cannot instantiate operator... exit!");
//					e1.printStackTrace();
//					return;
//				}
//				if (operator instanceof MTBOperatorControllable) {
//					// open controll window
//					OpControlGUI opController = new OpControlGUI();
//					try {
//						opController.open();
//					} catch (ALDOperatorException e1) {
//						// TODO Auto-generated catch block
//						e1.printStackTrace();
//					}
//				}
//				else {
//					ALDExecuteOperatorFrame executeOperator = new ALDExecuteOperatorFrame();
			}
		}

		// quit button has been pressed
		else if ( command.equals("quit") ) {
			this.setVisible( false);
			System.exit(0);
		}

		/* handle tree view options */

		else if (command.equals("viewApps")) {
			this.mainPanel.remove(this.opTreePane);
			this.buildUpOpNameTree(Level.APPLICATION);
			this.level = ALDAOperator.Level.APPLICATION;
			this.opTreePane = new JScrollPane(this.opTree);
			// insert scroll pane on top of main panel again
			this.mainPanel.add(this.opTreePane,0);
			this.mainPanel.setVisible(false);
			this.mainPanel.setVisible(true);
		}
		else if (command.equals("viewStd")) {
			this.mainPanel.remove(this.opTreePane);
			this.buildUpOpNameTree(Level.STANDARD);
			this.level = ALDAOperator.Level.STANDARD;
			this.opTreePane = new JScrollPane(this.opTree);
			// insert scroll pane on top of main panel again
			this.mainPanel.add(this.opTreePane,0);
			this.mainPanel.setVisible(false);
			this.mainPanel.setVisible(true);
		}
		
		/* display about box */
		else if (command.equals("showAbout")) {
						Object[] options = { "OK" };
						String msg = "<html>Alida - Advanced Library for Integrated Development<p>"
						    + "\t of Data Analysis Applications, Release 1.2<p><p>"
						    + "\u00a9 2010 - 2012   "
						    + "Martin Luther University Halle-Wittenberg<p><p>"
						    + "Institute of Computer Science, Faculty of Natural Sciences III<p><p>"
						    + "Email: alida@informatik.uni-halle.de <p>"
						    + "Internet: <i>www.informatik.uni-halle.de/alida</i><p>"
						    + "License: GPL 3.0, <i>http://www.gnu.org/licenses/gpl.html</i></html>";

						JOptionPane.showOptionDialog(null, new JLabel(msg),
						    "Information about Alida", JOptionPane.DEFAULT_OPTION,
						    JOptionPane.INFORMATION_MESSAGE, this.aboutIcon, options, options[0]);
		}
	}

	@Override
  public void valueChanged(TreeSelectionEvent event) {
		OpTreeNode node = (OpTreeNode)this.opTree.getLastSelectedPathComponent();
		// if sub-tree has been folded in, last selected node is null -
		// do nothing in that case
		if (node == null)
			return;
		if ( node.isOperator ) {
			this.opName = new String( node.fullName);
			this.opNameJText.setText( 
					this.opTree.getLastSelectedPathComponent().toString());
			this.opNameJText.setToolTipText( this.opName);
		}
	}

	/**
	 * Class for managing nodes of the tree view. 
	 */
	private class OpTreeNode extends DefaultMutableTreeNode {
		/**
		 * Indicates if node refers to an operator.
		 */
		boolean isOperator;

		/**
		 * Stores name of operator.
		 */
		String fullName;
		
		/**
		 * Parameterized constructor.
		 */
		OpTreeNode( String fName, String name, boolean isOp) {
			super( name);
			this.fullName = fName;
			this.isOperator = isOp;
		}

		/**
		 * Get full name of operator.
		 * @return	Full name of operator.
		 */
		String getFullName() {
			return this.fullName;
		}
	}

	/** 
	 * Node to build up a tree of the package structure of ALD operators. 
	 */
	private class Node {
		
		/**
		 * Children of this node.
		 */
		HashMap<String,Node>	children;
		
		/**
		 * Operator names associated with this node.
		 */
		LinkedList<String> opNames;
		
		/** 
		 * Full name from the root operator to this node.
		 */
		String fullName;

		/**
		 * Constructor.
		 * @param fName		Full name of root operator.
		 */
		Node( String fName) {
			this.children = new HashMap<String,Node>();
			this.opNames = new LinkedList<String>();
			this.fullName = fName;
		}

		/**
		 * Default constructor.
		 */
		Node() {
			this("");
		}

		/**
		 * Generates the tree.
		 * @param name	Operator name.
		 * @return	Created tree.
		 */
		OpTreeNode createTree ( String name) {
			OpTreeNode node = new OpTreeNode( this.fullName , name, false);

			java.util.Collections.sort( this.opNames);
			for ( String oName : this.opNames ) {
				OpTreeNode child = 
					new OpTreeNode( node.fullName + "." + oName, oName, true);
				node.add( child);
			}

			LinkedList<String> keysSorted = 
				new LinkedList<String>( this.children.keySet());
			java.util.Collections.sort( keysSorted);
			for ( String str : keysSorted ) {
				OpTreeNode child = this.children.get( str).createTree( str);
				node.add( child);
			}

			return node;
		}

		@SuppressWarnings("unused")
    private void print( String indent) {
			System.out.println( indent + "  fullName " + this.fullName);
			System.out.println( indent + "  opNames ");
			for ( String oName : this.opNames )
				System.out.println( indent + "         " + oName);
			for ( String str : this.children.keySet() ) {
				System.out.println( indent + str);
				this.children.get( str).print( indent + "    ");
			}
		}
	}

	/**
	 * Local MouseAdapter class.
	 * 
	 * @author posch
	 */
	class MyMouseAdapter extends MouseAdapter {
		
		/**
		 * Local action listener.
		 */
		ActionListener listener;
		
		/**
		 * Constructor.
		 * @param list	Action listener associated with adapter.
		 */
		MyMouseAdapter(ActionListener list) {
			this.listener = list;
		}
		
		@Override
		public void mousePressed(MouseEvent e) {
			int selRow = 
				ALDChooseOpNameFrame.this.opTree.getRowForLocation(e.getX(), e.getY());
//			TreePath selPath = ALDChooseOpNameFrame.this.opTree.getPathForLocation(e.getX(), e.getY());
			if(selRow != -1) {
				if(e.getClickCount() == 2) {
					//myDoubleClick(selRow, selPath);
					OpTreeNode node = (OpTreeNode)ALDChooseOpNameFrame.this.opTree.
																							getLastSelectedPathComponent();
					if ( node.isOperator ) {
						ALDChooseOpNameFrame.this.opName = new String( node.fullName);
						ALDChooseOpNameFrame.this.opNameJText.setText(
							ALDChooseOpNameFrame.this.opTree.getLastSelectedPathComponent().
																																	toString());
						ALDChooseOpNameFrame.this.opNameJText.setToolTipText( 
								ALDChooseOpNameFrame.this.opName);
						ActionEvent event = new ActionEvent( this, 1, "config");
						this.listener.actionPerformed( event);
					}
				}
			}
		}
	}
}
