/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5207 $
 * $Date: 2012-03-14 15:12:09 +0100 (Mi, 14. Mär 2012) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.Alida.operator.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import de.unihalle.informatik.Alida.operator.*;


import de.unihalle.informatik.Alida.dataio.ALDDataIOManagerSwing;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOManagerException;

/**
 * Frame with result data displayed on termination of an operator run.
 * 
 * @author Stefan Posch
 * @author Birgit Moeller
 */

public class ALDExecuteOperatorResultFrame extends JFrame 
	implements ActionListener {

	/**
	 * Local debug flag.
	 */
	private boolean debug = true;

	/**
	 * Map of compenents displayed in frame.
	 */
	private HashMap<ALDOpParameterDescriptor,JComponent> componentMap;
	
	/**
	 * Associated operator.
	 */
	private ALDOperator op = null;
	
	/**
	 * Configuration / control frame associated with the output.
	 */
	private ALDOperatorControlFrame inputFrame;

	/**
	 * Constructor.
	 * @param operator	Associated operator.
	 * @param iFrame		Associated control frame.
	 */
	public ALDExecuteOperatorResultFrame(ALDOperator operator, 
			ALDOperatorControlFrame iFrame) {
		
		this.op = operator;
		this.inputFrame = iFrame;
		this.componentMap = new HashMap<ALDOpParameterDescriptor, JComponent>();
		
		JPanel mainPanel = new JPanel();
		BoxLayout ylayout = new BoxLayout( mainPanel, BoxLayout.Y_AXIS );
		mainPanel.setLayout( ylayout);

		int numPanels = 0;
		// set up window

		for ( String pName : this.op.getOutInoutNames() ) {
			try {
				ALDOpParameterDescriptor descr= this.op.getParameterDescriptor( pName);
				Object value = this.op.getParameter( pName);

				if ( this.debug ) 
					System.out.println(
							"ALDExecuteOperatorFrame output value of parameter " 
							+ descr.getName() +	" = " + value);

				if ( value != null ) {
					JComponent guiElement = null;
					try {
						guiElement = ALDDataIOManagerSwing.getInstance().writeData( value);
						if ( guiElement != null ) {
							numPanels++;

							JPanel paramPanel = new JPanel( new FlowLayout());
							paramPanel.add( new JLabel( descr.getLabel()));
		
							this.componentMap.put( descr, guiElement);
							paramPanel.add( guiElement);
		
							mainPanel.add( paramPanel);
						}
					} catch (ALDDataIOManagerException exp) {
      			Object[] options = { "OK" };
      			JOptionPane.showOptionDialog(null, 
      					"No provider found! \n " + exp.getCommentString(), "Warning", 
      					JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
      					null, options, options[0]);
					}
				}
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}

		if ( numPanels == 0 ) {
			JLabel notifyLabel = 
				new JLabel("No additional result parameters to display...");
			notifyLabel.setAlignmentX(LEFT_ALIGNMENT);
			mainPanel.add(notifyLabel);
		}
			
		// ===========================================
		// show input parameters button
//		JButton showButton = new JButton("Show Input Parameter");
//		showButton.setActionCommand( "showParams");
//		showButton.addActionListener( this);
//		showButton.setBounds(50, 60, 80, 30);

		// quit button
		JButton quitButton = new JButton("Quit");
		quitButton.setActionCommand( "quit");
		quitButton.addActionListener( this);
		quitButton.setBounds(50, 60, 80, 30);

		JPanel buttonPanel = new JPanel();
//		buttonPanel.add(showButton);
		buttonPanel.add(quitButton);

		mainPanel.add(buttonPanel);

		JScrollPane scrollPane = new JScrollPane( mainPanel);
		this.add( scrollPane);

		this.setTitle("Operator Results for " + this.op.getName());
		if (numPanels == 0)
			this.getContentPane().setPreferredSize(new Dimension(400, 200));
		else
			this.getContentPane().setPreferredSize(new Dimension(400, 500));
		this.pack();
		this.setDefaultCloseOperation( DISPOSE_ON_CLOSE);
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
  public void actionPerformed(ActionEvent e) {

		// local variables
		String command = e.getActionCommand();

		// quit
		if (command.equals("quit")) {
			this.setVisible( false);
		}
		// show params
		else if (command.equals("showParams")) {
			System.out.println( "I will show you your inputs soon....");
//			this.inputFrame.setVisible( true);
		}
	}
}
