/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5373 $
 * $Date: 2012-04-05 14:54:40 +0200 (Do, 05. Apr 2012) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.MiToBo.core.dataio;



import de.unihalle.informatik.Alida.annotations.ALDDataIOProvider;
import de.unihalle.informatik.Alida.dataio.provider.swing.ALDDataIOSwing;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException.ALDDataIOProviderExceptionType;

import javax.swing.*;

import java.awt.geom.Point2D;
import java.lang.reflect.Field;
import java.util.Collection;
import java.util.LinkedList;

/**
 * Data I/O class for {@link java.awt.geom.Point2D.Double}.
 * 
 * @author moeller
 */
@ALDDataIOProvider

public class MTBAwtPoint2DDataIOSwing implements ALDDataIOSwing {

	@Override
	public Collection<Class<?>> providedClasses() {
		LinkedList<Class<?>> classes = new LinkedList<Class<?>>();
		classes.add(Point2D.Double.class);
		return classes;
	}

	@Override
	public JComponent createGUIElement(Field field, Class<?> cl, Object obj) {
		return new PointConfigPanel(obj);
	}

	@Override
	public void setValue(Field field, Class<?> cl, JComponent guiElement,
			Object value) throws ALDDataIOProviderException {
		if (!(guiElement instanceof PointConfigPanel))
			throw new ALDDataIOProviderException(
					ALDDataIOProviderExceptionType.INVALID_GUI_ELEMENT, 
					"MTBAwtPoint2DDataIO: setValue() received invalid GUI element!");
		if (!(value instanceof Point2D.Double))
			throw new ALDDataIOProviderException(
					ALDDataIOProviderExceptionType.INVALID_GUI_ELEMENT, 
					"MTBAwtPoint2DDataIO: setValue() received wrong object type!");
		((PointConfigPanel)guiElement).setValue((Point2D.Double)value);
		((PointConfigPanel)guiElement).updateUI();
	}

	@Override
	public Object readData(Field field, Class<?> cl, JComponent guiElement) 
	throws ALDDataIOProviderException {

		if (!(guiElement instanceof PointConfigPanel))
			throw new ALDDataIOProviderException(
					ALDDataIOProviderExceptionType.INVALID_GUI_ELEMENT, 
					"AwtPoint2DDataIO: readData() received invalid GUI element!");
		return ((PointConfigPanel)guiElement).readData();
	}

	@Override
	public JComponent writeData(Object obj) {
		PointConfigPanel p = new PointConfigPanel(obj);
		p.setEnabled(false);
		return p;
	}

	/**
	 * Panel to display and read 2D points.
	 * 
	 * @author moeller
	 */
	private class PointConfigPanel extends JPanel {
		
		/**
		 * Field for x-coordinate.
		 */
		private JTextField xField;
		/**
		 * Field for y-coordinate.
		 */
		private JTextField yField;
		
		/**
		 * Default constructor. 
		 * @param obj	Default object.
		 */
		public PointConfigPanel(Object obj) {
			double x=0,y=0;
			if (obj != null && obj instanceof Point2D.Double) {
				x = ((Point2D.Double)obj).x;
				y = ((Point2D.Double)obj).y;
			}
			this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			this.add(new JLabel("x = "));
			this.xField = new JTextField(10);
			this.xField.setText(Double.toString(x));
			this.add(this.xField);
			this.add(new JLabel("y = "));
			this.yField = new JTextField(10);
			this.yField.setText(Double.toString(y));
			this.add(this.yField);
		}
		
		/**
		 * Displays specified coordinates.
		 * @param p	Point to display.
		 */
		public void setValue(Point2D.Double p) {
			this.xField.setText(Double.toString(p.x));
			this.yField.setText(Double.toString(p.y));
			this.updateUI();
		}

		/**
		 * Gets the current coordinates.
		 * @return	Point configured in panel.
		 */
		public Point2D.Double readData() {
			double x = Double.valueOf(this.xField.getText()).doubleValue();
			double y = Double.valueOf(this.yField.getText()).doubleValue();
			return new Point2D.Double(x,y);
		}
		
	}

}
