/*
OME Bio-Formats package for reading and converting biological file formats.
Copyright (C) 2005-@year@ UW-Madison LOCI and Glencoe Software, Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*
LOCI Common package: utilities for I/O, reflection and miscellaneous tasks.
Copyright (C) 2005-@year@ Melissa Linkert, Curtis Rueden and Chris Allan.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*
LOCI Plugins for ImageJ: a collection of ImageJ plugins including the
Bio-Formats Importer, Bio-Formats Exporter, Bio-Formats Macro Extensions,
Data Browser and Stack Slicer. Copyright (C) 2005-@year@ Melissa Linkert,
Curtis Rueden and Christopher Peterson.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5428 $
 * $Date: 2012-04-12 18:53:45 +0200 (Do, 12. Apr 2012) $
 * $Author: moeller $
 * 
 */

/*
 * This class uses the Bio-Formats and LOCI-commons packages/libraries (see the two licenses at the top)
 * as well as source code from the LOCI-plugins package (see third license from the top)
 */

package de.unihalle.informatik.MiToBo.core.dataio;

import de.unihalle.informatik.Alida.dataio.provider.swing.ALDDataIOSwing;


import de.unihalle.informatik.Alida.annotations.ALDDataIOProvider;
import de.unihalle.informatik.MiToBo.core.datatypes.images.*;

import ij.ImageListener;
import ij.ImagePlus;

import javax.swing.*;

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.LinkedList;
import java.util.Vector;

/**
 * Implementation of ALDDataIOCmdline interface for MitoBo images.
 * 
 * @author moeller
 */
@ALDDataIOProvider

public class MTBImageDataIOSwing implements ALDDataIOSwing {
	
	/** 
	 * Index to number untitled images popped-up.
	 */
	private static int idxUntitled = 0;

	/**
	 * Default constructor.
	 */
	public MTBImageDataIOSwing() {
		// nothing to do here...
	}

	/**
	 * Interface method to announce class for which IO is provided for
	 * 
	 * @return	Collection of classes provided
	 */
	@Override
  public Collection<Class<?>> providedClasses() {
		LinkedList<Class<?>> classes = new LinkedList<Class<?>>();
		classes.add( MTBImage.class);
		classes.add( MTBImageByte.class);
		classes.add( MTBImageDouble.class);
		classes.add( MTBImageFloat.class);
		classes.add( MTBImageInt.class);
		classes.add( MTBImageRGB.class);
		classes.add( MTBImageShort.class);
		classes.add( ImagePlus.class);
		return classes;
	}

	@Override
	public JComponent createGUIElement(Field field, Class<?> cl, Object obj) {
		String name = null;
		if (obj != null) {
			if (obj instanceof MTBImage) {
				MTBImage img = (MTBImage)obj;
				name = img.getTitle();
			}
			else if (obj instanceof ImagePlus) {
				ImagePlus imgp = (ImagePlus)obj;
				name = imgp.getTitle();
			}
		}
		return new JComboBoxImage(name);
	}

	@Override
  public void setValue(Field f, Class<?> cl, JComponent guiElem, Object value){
		JComboBoxImage box  = ((JComboBoxImage)guiElem);
		box.setValue(value);
  }

	@Override
	public Object readData(Field field, Class<?> cl, JComponent guiElement) {
		JComboBoxImage box  = ((JComboBoxImage)guiElement);
		return box.readData(field, cl);
	}

	@Override
	public JComponent writeData(Object obj) {
		if (obj instanceof MTBImage) {
			MTBImage img = ((MTBImage)obj);

			if ( img.getTitle().equals("") ) {
				img.setTitle( "Untitled-" + idxUntitled);
				idxUntitled++;
			}

			((MTBImage)obj).show();
			return null;
		}
		else if (obj instanceof ImagePlus) {
			ImagePlus imgp = (ImagePlus)obj;
			imgp.show();
		}
		return null;
	}
	
	/**
	 * Class for handling images opened in GUI.
	 * 
	 * @author moeller
	 */
	private class JComboBoxImage extends JComboBox implements ImageListener {

		/**
		 * Recently selected image.
		 */
		private String selectedImage = "none selected";

		/**
		 * Default constructor.
		 * 
		 * @param obj		Optional name of default object, may be null.
		 */
		public JComboBoxImage(String objName) {
			super();
			// register in ImageJ as listener
			ImagePlus.addImageListener(this);
			this.updateImageList(objName);
		}

	  /**
	   * Selects the given image object in combobox.
	   * @param value 	Image object to be selected.
	   */
	  public void setValue(Object value){
	  	if (value == null)
	  		return;
			if (value instanceof MTBImage) {
				MTBImage img = (MTBImage)value;
				img.show();
				updateImageList(img.getTitle());
			}
			else if (value instanceof ImagePlus) {
				ImagePlus imgp = (ImagePlus)value;
				imgp.show();
				updateImageList(imgp.getTitle());
			}
	  }

		/**
		 * Function to get selected image from GUI.
		 * 
		 * @param field	Field of requested object.
		 * @param cl		Class of selected object.
		 * @return	Image object.
		 */
		@SuppressWarnings("unused")
    public Object readData(Field field, Class<?> cl) {
			ImagePlus selectedImg = null;
			
			// get currently selected item
			String imgName = (String)(this.getSelectedItem());

			// find image with selected title in GUI
			int[] ids = ij.WindowManager.getIDList();
			if (ids == null)
				return null;
			for ( int id : ids ) {
				ImagePlus img = ij.WindowManager.getImage( id);
				if ( imgName.equals( img.getTitle()) ) {
					selectedImg = img;
				}
			}
			// safety check, if something went wrong, return nothing
			if ( selectedImg == null )
				return null;
			
			// remember the selected image name
			this.selectedImage = imgName;
			
			// return result
			if (cl.equals(ImagePlus.class))
				return selectedImg;
			return MTBImage.createMTBImage( selectedImg);
		}

		/**
		 * Updates image list and GUI.
		 * @param preselect		Optional name of object to be pre-selected.
		 */
		private void updateImageList(String preselect) {
			Vector<String> allImgNames = new Vector<String>();
			allImgNames.add("none selected");
			int idx = 0;

			// remember which image to be selected
			this.selectedImage = "none selected";
			
			// get list of current images
			int[] ids = ij.WindowManager.getIDList();
			if ( ids != null ) {
				int i = 0;
				for ( int id : ids ) {
					ImagePlus img = ij.WindowManager.getImage( id);
					allImgNames.add( img.getTitle());
					i++;
					// check if image name matches desired pre-selection
					if (preselect != null && preselect.equals( img.getTitle()) ) {
						idx = i;
						this.selectedImage = preselect;
					}
				}
			}
			this.removeAllItems();
			for (String image: allImgNames)
				this.addItem(image);
			this.setSelectedIndex(idx);
		}

		@Override
		public void imageClosed(ImagePlus arg0) {
			this.updateImageList(this.selectedImage);
		}

		@Override
		public void imageOpened(ImagePlus arg0) {
			this.updateImageList(this.selectedImage);
		}

		@Override
		public void imageUpdated(ImagePlus arg0) {
			// here nothing needs to be done...
		}
	}
}
