/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5457 $
 * $Date: 2012-04-17 15:36:33 +0200 (Di, 17. Apr 2012) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.MiToBo.core.dataio;



import de.unihalle.informatik.Alida.annotations.ALDDataIOProvider;
import de.unihalle.informatik.Alida.dataio.provider.helpers.ALDTableWindow;
import de.unihalle.informatik.Alida.dataio.provider.swing.ALDDataIOSwing;
import de.unihalle.informatik.Alida.exceptions.*;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException.ALDDataIOProviderExceptionType;
import de.unihalle.informatik.MiToBo.apps.cells2D.*;
import de.unihalle.informatik.MiToBo.gui.MTBTableModel;

import javax.swing.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.Field;
import java.util.*;

/**
 * Class for loading/saving MTBTableModel data objects.
 * 
 * @author moeller
 */
@ALDDataIOProvider

public class MTBTableModelDataIOSwing implements ALDDataIOSwing {

	/**
	 * Interface method to announce class for which IO is provided for.
	 * 
	 * @return  Collection of classes provided.
	 */
	@Override
	public Collection<Class<?>> providedClasses() {
		LinkedList<Class<?>> classes = new LinkedList<Class<?>>();
		classes.add(MTBTableModel.class);
		classes.add(Mica2DTableModel.class);
		return classes;
	}
	
	/** 
	 * Generic reading of table models.
	 */
	/* (non-Javadoc)
	 * @see de.unihalle.informatik.Alida.helpers.ALDDataIOSwing#createGUIElement(java.lang.Class, java.lang.Object)
	 */
	@Override
	public JComponent createGUIElement(Field field, Class<?> cl, Object obj) {
		return new JLabel("Table model inputs are not supported yet!");
	}

	@Override
  public void setValue(Field field, Class<?> cl, JComponent guiElement,
      Object value) throws ALDDataIOProviderException {
	  // TODO Auto-generated method stub
	  
  }

	@Override
	public Object readData(Field field, Class<?> cl, JComponent guiElement) {
//		if (!(guiElement instanceof CollectionConfigButton))
//			throw new ALDDataIOProviderException(
//					ALDDataIOProviderExceptionType.INVALID_GUI_ELEMENT, 
//					"CollectionDataIO: readData received invalid GUI element!");
//		return ((CollectionConfigButton)guiElement).readData(field, cl);
		return null;
	}

	@Override
	public JComponent writeData(Object obj) throws ALDDataIOProviderException {
		if (!(obj instanceof MTBTableModel))
			throw new ALDDataIOProviderException(
					ALDDataIOProviderExceptionType.OBJECT_TYPE_ERROR, 
					"MTBTabelModelDataIO: object to write has wrong type!");
		// return a button to show a window with the elements
		return new TableModelShowButton(obj);
	}

//	/**
//	 * GUI element for configuring collections.
//	 * <p>
//	 * This button has a collection configuration window attached to it 
//	 * where specific data is stored and accessable.
//	 * 
//	 * @author moeller
//	 */
//	private class CollectionConfigButton extends JButton {
//
//		/**
//		 * Collection configuration window.
//		 */
//		private CollectionConfigWindow confWin;
//
//		/**
//		 * Constructor.
//		 * 
//		 * @param field	Field of collection.
//		 * @param cl	Class of collection.
//		 * @param obj	Default object.
//		 */
//		public CollectionConfigButton(Field field, Class<?> cl, Object obj) {
//			super("Configure Collection...");
//			this.confWin = new CollectionConfigWindow(field, cl, obj);
//			this.setActionCommand("configButtonPressed");
//			this.addActionListener(this.confWin);
//		}
//		
//		/**
//		 * Gets the data from the configuration window.
//		 * 
//		 * @param field	Field of collection.
//		 * @param cl	Class of collection.
//		 * @param obj	Default object.
//		 * @return Current data.
//		 */
//		public Object readData(Field field, Class<?> cl) {
//			return this.confWin.readData(field, cl);
//		}
//	}	
	
	/**
	 * GUI element for displaying table model data.
	 * <p>
	 * This button opens a window with a table containing the data.
	 * 
	 * @author moeller
	 */
	private class TableModelShowButton extends JButton 
		implements ActionListener{

		/**
		 * Data to be displayed.
		 */
		private MTBTableModel data = null;

		/**
		 * Associated window.
		 */
		private ALDTableWindow tabwin = null;
		
		/**
		 * Constructor.
		 */
		public TableModelShowButton(Object obj) {
			super("Show data...");
			this.setActionCommand("showButtonPressed");
			this.addActionListener(this);
			this.data = (MTBTableModel)obj;
		}
		
		@Override
		public void actionPerformed(ActionEvent e) {
			String cmd = e.getActionCommand(); 
			if (cmd.equals("showButtonPressed")) {
				if (this.data != null) {
					if (this.tabwin == null) {
						this.tabwin = new ALDTableWindow(this.data);
						this.tabwin.setTitle("MTBTableModel data:");
						this.tabwin.setVisible(true);
					}
					else
						this.tabwin.setVisible(true);
				}
				else {
					Object[] options = { "OK" };
					JOptionPane.showOptionDialog(null, 
							"There is no data to display!", "Warning", 
							JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
							null, options, options[0]);
				}	
			}	
		}
	}
}
