/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5376 $
 * $Date: 2012-04-10 10:35:04 +0200 (Di, 10. Apr 2012) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.MiToBo.core.operator.gui;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;

import ij.IJ;
import loci.common.StatusEvent;
import loci.common.StatusListener;
import loci.common.StatusReporter;


import de.unihalle.informatik.Alida.operator.gui.ALDOperatorControlFrame;
import de.unihalle.informatik.Alida.operator.gui.OnlineHelpDisplayer;

/**	This frame is used to display and modify the input parameters of an ALDOperator.
 *  If the operator is configured, i.e. iput parameters set, it may be executed
 *  using an ALDExecuteOperatorFrame. This JFrame is free to decide, what executing
 *  precisily means, usually it is to start the operator via its runOp method
 *  and display or store the results appropriately.
 *  <p>
 *  There are thres methodes which control the handling of each input parameter
 *  of the operator to a certain degree.  These are <code>resetCreateInParameterPanel</code>,
 *  <code>registerDescriptor</code>, and <code>setDefaultValue</code>.
 *  This customizing requires quit some knowledge
 *  of the implementation of <code></code>.
 * @author Stefan Posch
 */

public class MTBOperatorControlFrame extends ALDOperatorControlFrame 
	implements StatusListener {

	@SuppressWarnings("unused")
  private boolean debug = false;

	/**
	 * Construct a JFrame to intantiate and configure an operator <code>opName</code>
   * and use <code>executeOperator</code> to execute this operator if requested by the user.
	 *
	 * @param	opName
	 * @param	executeOperator
	 */
	public MTBOperatorControlFrame(String opName) {
		super(opName);
		this.titleString = "MTBOperatorControlFrame: " + this.op.getName();
		this.setTitle(this.titleString);
		Class<?>[] classes = (this.op.getClass().getInterfaces());
		for (Class<?> c: classes) {
			if (c.equals(StatusReporter.class)) {
				((StatusReporter)this.op).addStatusListener(this);
			}
		}
	}
	
	@Override
	protected JMenu generateHelpMenu() {
		JMenu helpM = new JMenu("Help");
		JMenuItem itemHelp = new JMenuItem("Online Help");
		itemHelp.addActionListener(OnlineHelpDisplayer.getHelpActionListener(
				itemHelp,	this.op.getClass().getName()));
		JMenuItem itemAbout = new JMenuItem("About MiToBo");
		itemAbout.setActionCommand("helpM_about");
		itemAbout.addActionListener(this);
		helpM.add(itemHelp);
		helpM.add(itemAbout);
		return helpM;
	}

	@Override
  public void addParamConfigPanel() {		
		this.operatorParameterPanel = 
		new MTBOperatorParameterPanel(this.op, this.displayMode, true);
//	this.operatorParameterPanel.setTopLevelFlag(true);
	JScrollPane scrollPane = new JScrollPane(this.operatorParameterPanel);
	this.inputPanel.add(scrollPane);

//		this.operatorIOObj = new MTBOperatorDataIOSwing();
//		this.operatorIOObj.setTopLevelFlag(true);
//		this.operatorParameterPanel = 
//			this.operatorIOObj.getConfigPanel(this.op, this.displayMode);
////		this.operatorParameterPanel.setPreferredSize(new Dimension(1500,1500));
//
//		// TODO wrap up with a scrolling panel - but does not work!!!
//		JScrollPane scrollPane = new JScrollPane(this.operatorParameterPanel);
////		this.getContentPane().add(scrollPane, BorderLayout.CENTER);
////		this.inputPanel.add(this.operatorParameterPanel);
//		this.inputPanel.add(scrollPane);
	}
	
	@Override
	public void statusUpdated(StatusEvent e) {
		IJ.showStatus(e.getStatusMessage());
		IJ.showProgress(e.getProgressValue(), e.getProgressMaximum());
	}
}
