package de.unihalle.informatik.MiToBo.math;




/**
 * Math class with functions that are not provided by Java's Math class.
 * @author Oliver Gress
 *
 */

public class MathX {

	
	/**
	 * Natural logarithm of the sum of two values P1 and P2 when only their natural logarithms log(P1) and log(P2) are given. 
	 * @param logP1 natural logarithm of P1: log(P1)
	 * @param logP2 natural logarithm of P2: log(P2)
	 * @return log( P1 + P2 )
	 */
	public static double logSumP(double logP1, double logP2) {
		if (logP1 == 0.0)
			return logP2;
		if (logP2 == 0.0)
			return logP1;
		if (logP1 < logP2)
			return logP2 + Math.log1p(Math.exp(logP1 - logP2));
		else
			return logP1 + Math.log1p(Math.exp(logP2 - logP1));
	}
	
	/**
	 * Binomial coefficient of n over k
	 * @param n
	 * @param k
	 * @return
	 */
	public static double binomial(int n, int k) {
		if (k < 0 || n < 0 || n < k)
			throw new IllegalArgumentException("MathX.binomial(int n, int k): Following properties must be satisfied: n >= 0, k >= 0, k <= n.");
		
		if (k == 0)
			return 1;
		else if (k == 1)
			return n;
		else if (k == n)
			return 1;
		else {
			int k_ = k;
			
			if (k_ > n - k)
				k_ = n - k;
			
			double b1 = 1.0;
			double b2 = 1.0;
			for (int i = 0; i < k_; i++) {
				b1 *= (n-i);
				b2 *= (i+1);
			}
			
			return b1/b2;
		}	
	}
	
}
