/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5495 $
 * $Date: 2012-04-19 16:00:48 +0200 (Do, 19. Apr 2012) $
 * $Author: posch $
 * 
 */

package de.unihalle.informatik.MiToBo.segmentation.levelset.nonPDE;


import java.io.PrintStream;



/** An abstract class for energies for a nonPDF levelset approach
 * based on curvature energies of the phases boundaries.
 */


public abstract class CurvatureEnergy extends MTBGenericEnergyNonPDE
{
	/** Weight of the curvature term
	 */
	protected double kappa = 1.0; 

	/** Bit mask for curvature energies for debugging purposes.
	 * The value has to be coordinated with other bit masks.
	 */
    protected final int FLAG_CURVATURE = 8; // flag for curvature term

	/**
	 * Constructor
	 * 
	 */
	public CurvatureEnergy() {
		this( 1.0);
	}

	/**
	 * Constructor
	 * 
	 * @param kappa	weight of the curvature term
	 */
	public CurvatureEnergy(double kappa) {
		this.kappa = kappa;
		this.name = new String( "Curvature energy");
	}

	/**
	 */
	@Override
	public  String 	toString() {
        return name + "kappa = " + kappa;
    }

	/**
	 * Create a proper curvature energy asked for (2D vs 3D).
	 *<br>
	 * In the future more types, e.g. various methods 
	 * may be available
	 *
	 * @param	kappa	weight of curvature term
	 * @param	threeD	
	 */
	public static CurvatureEnergy createEnergy( MTBLevelsetMembership ls, double kappa, boolean threeD) {
		if ( threeD ) {
			return new CurvatureEnergy3D( kappa);
		} else {
			return new CurvatureEnergy2D( ls, kappa);
		}
	}

	/**
	 * Create a proper curvature energy asked for (2D vs 3D).
	 * If sizeZ is one than 2D, 3D otherwise.
	 *<br>
	 * In the future more types
	 * may be available
	 *
	 * @param	kappa weight of curvature term
	 * @param	ls Levelset function
	 * @return	curvature energy
	 */
	public static CurvatureEnergy createEnergy(MTBLevelsetMembership ls, double kappa) {
		if ( ls.getSizeZ() != 1 ) {
			return new CurvatureEnergy3D( kappa);
		} else {
			return new CurvatureEnergy2D( ls, kappa);
		}
	}

}
