/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5495 $
 * $Date: 2012-04-19 16:00:48 +0200 (Do, 19. Apr 2012) $
 * $Author: posch $
 * 
 */

// TODO: lookup of pixels in contour is sequqntial!!
package de.unihalle.informatik.MiToBo.segmentation.levelset.nonPDE;

import java.util.*;
import java.io.*;
import java.lang.Math;

import de.unihalle.informatik.MiToBo.io.files.*;
import de.unihalle.informatik.MiToBo.core.datatypes.images.*;
import de.unihalle.informatik.MiToBo.core.datatypes.images.MTBImage.MTBImageType;



/** Implements the a curvature energy for 2D images.
 */



public class CurvatureEnergy2D extends CurvatureEnergy {

	private final int numPhases; // number of phases including background

	private final int sizeX;
	private final int sizeY;

	private Vector<LinkedList<CoordInt2D>> contours;
	//private LinkedList<CoordInt2D> contours[];

	/**
	 * constructor
	 * 
	 * @param kappa
	 */
	public CurvatureEnergy2D( MTBLevelsetMembership lsm, double kappa) {
		super( kappa);
		this.name = new String( "Curvature energy (2D)");

		this.sizeX = lsm.getSizeX();
		this.sizeY = lsm.getSizeY();

		this.numPhases = lsm.getNumPhases();
		this.contours = new Vector<LinkedList<CoordInt2D>>(numPhases+1);
		for ( byte p = 0 ; p <= numPhases ; p++ ) 
			contours.add( null);

		this.estimateParams( lsm);
	}

	/**
	 */
	public double deltaE(int x, int y, int z, byte newPhase, MTBLevelsetMembership lsm) {

		int idx = -2;

		if ( lsm.getPhase( x,y) != MTBLevelsetMembership.BG_PHASE ) {
			 idx = lookupContour( x, y, lsm);
		}

		double delta = 0.0;

		if ( (debug & FLAG_CURVATURE) != 0 ) {
			System.out.println( name +"#deltaE for (" + x + "," + y + "), idx in contour: " + idx);
		}

		return kappa * delta;
	}

	/**
	*/

	public double E(MTBLevelsetMembership lsm)
	{
		double sum = 0.0;
			
		// remind; we are in 2D
		for(int y = 0; y < sizeY; y++) {
			for(int x = 0; x < sizeX; x++) {
					sum += E( x, y, lsm);
			}
		}
		return sum;
	}
	
	/** Get energy term a pixel (x,y) for LSM. 2D only
	 * @param x
	 * @param y
	 * @param LSM
	 * @return energy
	*/

	public double E( int x, int y, MTBLevelsetMembership lsm) {
		return kappa;
    }


	/** 
	 */
	public void updateParams(int x, int y, int z, byte newPhase, MTBLevelsetMembership lsm) {
	}

	/** 
	 */
	protected void estimateParams(MTBLevelsetMembership lsm) {
		for ( byte p = MTBLevelsetMembership.BG_PHASE+1 ; p <= numPhases ; p++ ) {
			contours.setElementAt( findContour( p, lsm), p);
		}

		toImage( contours, "contoursFollowed.tif");
	}

	// =======================================================================
	// PRIVATE helpers: contour following

	/** neighborhood array of neighbors to reuse for efficiency.
	 */
	boolean nb[] = new boolean[8];

	/** Find (first) outer contour of phase in a levelset function.
	 * 
	 * @param phase		
	 * @param ls		Levelset function
	 */
	LinkedList<CoordInt2D> findContour( byte phase, MTBLevelsetMembership lsm) {
		LinkedList<CoordInt2D> ctr = new LinkedList<CoordInt2D>();

		for(int y = 0; y < sizeY; y++) {
			for(int x = 0; x < sizeX; x++) {
				if ( lsm.getPhase( x,y) == phase ) {
					if ( (debug & FLAG_CURVATURE) != 0 ) {
						System.out.println( "CurvatureEnergy2D#findContour found first pixel for phase " +
							phase + " @(" + x + "," + y + ")");
					}

					CoordInt2D start = new CoordInt2D( x, y);
					CoordInt2D second = new CoordInt2D();
					CoordInt2D current = new CoordInt2D();
					CoordInt2D previous = new CoordInt2D();
					CoordInt2D next = new CoordInt2D();
					int dir = 0;

					dir = findNext( dir, start, second, lsm);
					ctr.add( new CoordInt2D( second));
					//System.out.println( " add: " + second + " got dir = " + dir);

					previous.set( start);
					current.set( second);

					if ( ! start.equals( current) ) {
						while ( true ) {
							dir = findNext( (dir+6)%8, current, next, lsm);
							previous.set( current);
							current.set( next);

							if (  previous.equals( start) && current.equals( second) ) {
								//System.out.println( "Contour");
								//Iterator<CoordInt2D> itr = ctr.iterator();
								//while ( itr.hasNext() ) {
									//System.out.println( itr.next());
								//} 
								return ctr;
							} else {
								//System.out.println( " add: " + next + " got dir = " + dir);
								ctr.add( new CoordInt2D( next));
							}
						}
					}
				}
			}
		}

		return ctr;
	}

	/** Find next contour point
	 *
	 * @param dir	direction
	 * @param current	current contour point
	 * @param next	next contour point: returned!!
	 * @return new direction
	 */
	private int findNext( int dir, CoordInt2D current, CoordInt2D next, MTBLevelsetMembership lsm) {
		byte phase = lsm.getPhase( current.x, current.y);
		for ( int i = 0; i <= 6 ; i++ ) {
			next.x = current.x + deltaX[dir];
			next.y = current.y + deltaY[dir];

			if ( lsm.getPhaseCheckXY( next.x, next.y) != phase ) {
				dir = (dir+1) % 8;
			} else {
				int nextDir = (dir+1)%8;

				// if clause is to get contour points 8 connected to the background
				if ( (dir%2) == 1 &&
                     lsm.getPhaseCheckXY( current.x + deltaX[nextDir], current.y + deltaY[nextDir]) == phase ) {
					next.x = current.x + deltaX[nextDir];
					next.y = current.y + deltaY[nextDir];
					return nextDir;
				} else {
					return dir;
				}
			}
				
		}

		return -1; // should never happen

	}

	/** Lookup contour and index in this contour for a pixel (x,y)
	 *
	 * @param x
	 * @param y
	 * @param lsm levelset function
	 */
	private int lookupContour( int x, int y, MTBLevelsetMembership lsm) {
		LinkedList<CoordInt2D> ctr = contours.elementAt( lsm.getPhase( x,y));
		return ctr.indexOf( new CoordInt2D( x, y));
	}
	
	// =======================================================================
	// PRIVATE helpers for neighborhoods

	/** delta for x-coordinates for a 8 neightborhood.
	 *
	 * <table border="1">
  	 *   <tr> <th>5</th> <th>6</th> <th>7</th> </tr> 
	 *   <tr> <th>4</th> <th>x</th> <th>0</th> </tr> 
	 *   <tr> <th>3</th> <th>2</th> <th>1</th> </tr> 
	 * </table>
	 */
	private int deltaX[] = new int[] {1,1,0,-1,-1,-1,0,1};

	/** delta for y-coordinates for a 8 neightborhood.
	 *
	 * <table border="1">
  	 *   <tr> <th>5</th> <th>6</th> <th>7</th> </tr> 
	 *   <tr> <th>4</th> <th>x</th> <th>0</th> </tr> 
	 *   <tr> <th>3</th> <th>2</th> <th>1</th> </tr> 
	 * </table>
	 */
	private int deltaY[] = new int[] {0,1,1,1,0,-1,-1,-1};

	/** Get the 8 neighborhood of pixel (x,y) and indicate if the same phase.
	 * If a pixel has the same phase as (x,y) this results in true, i.e. NOT foreign.
	 * Reuse the array nb passed argument.
	 * 
	 * @param x
	 * @param y
	 * @param lsm	levelset function
	 * @param nb	neighborhood array of size 8 to reuse, i.e. return the results in
	 */
	private void get8Neighborhood( int x, int y, MTBLevelsetMembership lsm, boolean nb[]) {
		byte phase = lsm.getPhase( x, y);
		for ( int n=0 ; n < 8 ; n++ ) {
			if ( lsm.getPhaseCheckXY( x+deltaX[n], y+deltaY[n]) == phase )
				nb[n] = true;
			else
				nb[n] = false;
		}
	}

	/** Get the 8 neighborhood of pixel (x,y), indicate if the same phase,
	 * and remove 8 neighbors not connected to a 4 neighbor.
	 * If a pixel has the same phase as (x,y) this results in true, i.e. NOT foreign.
	 * Reuse the array nb passed argument.
	 * 
	 * @param x
	 * @param y
	 * @param lsm	levelset function
	 * @param nb	neighborhood array of size 8 to reuse, i.e. return the results in
	 */
	private void get8Neighborhood4Connected( int x, int y, MTBLevelsetMembership lsm, boolean nb[]) {
		get8Neighborhood( x, y, lsm, nb);
		removeIsolated8Neighbors( nb);
	}

	/** Remove 8 neighbors which are not connected to a 4 neighbor.
	 * 
	 * @param nb	neighborhood array of size 8 to reuse, i.e. return the results in
	 */
	private void removeIsolated8Neighbors( boolean nb[]) {
		for ( int n=1 ; n < 8 ; n += 2 ) {
			if ( nb[n] == true && nb[(n-1)%8] == false && nb[(n+1)%8] )
				nb[n] = false;
		}
	}

	/** Is the neightborhood empty, i.e. there are no neighbors?
	 * 
	 * @param nb	neighborhood array of size 8 
	 */
	private boolean isEmpty( boolean nb[]) {
		for ( int n=0 ; n < 8 ; n ++ ) {
			if ( nb[n] == true ) return false;
		}
		return true;
	}

	/** Find last 4 connected neighbor in 8 neighborhood in counter clockwise direction.
	 * The neighborhood is assumed to be valid for pixel (x,y)
	 * We assume one connected component of neighbors. If there are more an
	 * arbitray connected component is used.
	 * <b>Note</b> We assume at least ONE 4 connected neighbor, otherwise we have
	 * an endless loop.
	 * 
	 * @param x
	 * @param y
	 * @param nb	neighborhood array of size 8 to reuse, i.e. return the results in
	 * @retrun last 4 connected neighbor
	 */
	private CoordInt2D findLast4Neighbor( int x, int y, boolean nb[]) {
		int n;

		// find first non neighbor
		n=7;
		while ( nb[n] == true ) n = (n-1)%8;
		// find next neighbor, i.e. first neighbor of connected component
		n = (8+n-1)%8;
		while ( nb[n] == false ) n = (8+n-1)%8;

		// go on and find last neighbor in this CC
		while ( nb[(8+n-1)%8 ] == true ) n = (8+n-1)%8;

		// backup one step if we found an 8 neighbor as last pixel in CC
		if ( (n%2) == 1 ) n = (n+1)%8;

		// find the last 4 connected neighbor of this component
		return new CoordInt2D( x + deltaX[n], y + deltaY[n]);
	}

	private void printNB( boolean nb[]) {
		System.out.println ( ">> " + (nb[5]?1:0) + "\t" + (nb[6]?1:0) + "\t" + (nb[7]?1:0));
		System.out.println ( "   " + (nb[4]?1:0) + "\t" + 'x' + "\t" +         (nb[0]?1:0));
		System.out.println ( "   " + (nb[3]?1:0) + "\t" + (nb[2]?1:0) + "\t" + (nb[1]?1:0));
	}

	private void toImage( Vector<LinkedList<CoordInt2D>> ctrs, String name) {
		MTBImage img =  MTBImage.createMTBImage( sizeX, sizeY, 1, 1, 1, MTBImageType.MTB_BYTE);

		for ( byte p = MTBLevelsetMembership.BG_PHASE+1 ; p <= numPhases ; p++ ) {
			if ( ctrs.elementAt(p) != null ) {
				Iterator<CoordInt2D> itr = ctrs.elementAt(p).iterator();
				while ( itr.hasNext() ) {
					CoordInt2D c = itr.next();
					img.putValueInt( c.x, c.y, 0, 255);
				} 
			} 
		} 

		try {
			//ImageSave writer= new ImageSave( name, img, true, true);
			ImageWriterMTB writer = new ImageWriterMTB( img, name);
			writer.runOp( null);
		} catch ( Exception e) {
			e.printStackTrace();
		}
	}

	// =======================================================================
	// PRIVATE CLASSES

	/**
	 * Private class to just hold integer 2D coordinates
	 */
	private class CoordInt2D extends Object {
		int x,y;

		CoordInt2D( int x, int y) {
			this.x = x;
			this.y = y;
		}

		CoordInt2D( CoordInt2D c) {
			this.x = c.x;
			this.y = c.y;
		}

		CoordInt2D( ) {
			this.x = -1;
			this.y = -1;
		}

		/** Equal coordinates?
		 */
		@Override
		public boolean equals( Object c) {
			CoordInt2D cc = (CoordInt2D)c;
			return x == cc.x && y == cc.y;
		}

		/** 
		 */
		@Override
		public int hashCode() {
			return y*sizeX + x;
		}

		/** set coordinates from argument
		 */
		private CoordInt2D set( CoordInt2D c) {
			this.x = c.x;
			this.y = c.y;
			return this;
		}

		/** print 
		 */
		private void print( CoordInt2D c) {
			System.out.print( "(" + c.x + "," + c.y + ")");
		}

		/** toString
		 */
		public String toString( ) {
			return new String( "(" + x + "," + y + ")");
		}
	}

}
