/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5495 $
 * $Date: 2012-04-19 16:00:48 +0200 (Do, 19. Apr 2012) $
 * $Author: posch $
 * 
 */

package de.unihalle.informatik.MiToBo.segmentation.levelset.nonPDE;


import java.lang.Math;
import java.io.PrintStream;



/** Implements the a curvature energy for 3D images.
 */



public class CurvatureEnergy3D extends CurvatureEnergy {

	/**
	 * constructor
	 * 
	 * @param kappa
	 */
	public CurvatureEnergy3D(double kappa) {
		super( kappa);
		this.name = new String( "Curvature energy (3D)");
	}

	/**
	 */
	public double deltaE(int x, int y, int z, byte newPhase, MTBLevelsetMembership LSM) {
		if ( (debug & FLAG_CURVATURE) != 0 ) System.out.println( "deltaE @(" + x + "," + y + ")");

		double delta = 0.0;

		return kappa * delta;
	}

	/**
	*/

	public double E(MTBLevelsetMembership LSM)
	{
		double sum = 0.0;
			
		// remind; we are in 3D
		for(int z = 0; z < LSM.getSizeZ(); z++) {
			for(int y = 0; y < LSM.getSizeY(); y++) {
				for(int x = 0; x < LSM.getSizeX(); x++) {
					sum += E( x, y, z, LSM);
				}
			}
		}
		return sum;
	}
	
	/** Get energy term a voxel (x,y,z) for LSM. 
	 * @param x
	 * @param y
	 * @param z
	 * @param LSM
	 * @return energy
	*/

	public double E( int x, int y, int z, MTBLevelsetMembership LSM) {
		return kappa;
    }

}
