/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5495 $
 * $Date: 2012-04-19 16:00:48 +0200 (Do, 19. Apr 2012) $
 * $Author: posch $
 * 
 */

package de.unihalle.informatik.MiToBo.segmentation.levelset.nonPDE;


import de.unihalle.informatik.Alida.annotations.ALDDerivedClass;
import java.lang.Math;
import java.io.PrintStream;



/** Implements the (standard) length energy using forward differences
 *   for 2D images and a multi phase levelset function.
 * Length is approximated using forward differences of the heaviside function
 */


@ALDDerivedClass

public class MTBLengthEnergyFW2DMPNonPDE extends MTBLengthEnergyFW2DNonPDE {
	private final double SQRT_2 = Math.sqrt(2.0);

	/**
	 * constructor
	 * 
	 * @param mu
	 */
	public MTBLengthEnergyFW2DMPNonPDE(double mu) {
		super( mu);
		this.name = new String( "Length energy (forward differences, 2D, multi phase)");
	}

	/**
	 * constructor
	 */
	public MTBLengthEnergyFW2DMPNonPDE() {
		this.name = new String( "Length energy (forward differences, 2D, multi phase)");
	}

	/**
	 * calculate the gradient magnitude of the foreward difference of the heavyside function
	 * corresponding to the level set for one pixel
     * 
	 * 
	 * @param a	heaviside functtion at (x,y,z)
	 * @param b heaviside functtion at (x+1,y,z)
	 * @param c heaviside functtion at (x,y+1,z)
	 * @return
	 */
	// TODO: check whether correct for multiphase
	protected double gradMagHeaviside(double a, double b, double c) {
        int diff1 = ( a == b ) ? 0 : 1;
        int diff2 = ( a == c ) ? 0 : 1;

        int sum =  diff1+diff2;
        if ( sum == 2 ) 
			return scaleFactor * SQRT_2;
		else
			return scaleFactor * sum;
	}

}
