/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5495 $
 * $Date: 2012-04-19 16:00:48 +0200 (Do, 19. Apr 2012) $
 * $Author: posch $
 * 
 */

package de.unihalle.informatik.MiToBo.segmentation.levelset.nonPDE;


import java.lang.Math;
import java.io.PrintStream;



/**
 * A class to implement the size energy of the Chan-Vese energy.
 *
 * @author Stefan Posch
 *
 */


public class MTBSizeEnergyNonPDE extends MTBGenericEnergyNonPDE {
	private final double nu;
	
	/** 
	 * Constructor for size energy with weight nu.
	 *
	 * @param nu weight of size energy
	 */
	public MTBSizeEnergyNonPDE(double nu) {
		this.name = new String( "MTBSizeEnergyNonPDE");
		this.nu = nu;
	}

	@Override
	public double deltaE(int x, int y, int z, MTBLevelsetMembership LSM) {
		byte val = LSM.getPhase(x, y, z);
		if(val <= MTBLevelsetMembership.BG_PHASE) 
			return this.nu;
		else 
			return (-1.0) * this.nu;
	}
	
	@Override
	public double deltaE(int x, int y, int z, byte newPhase, MTBLevelsetMembership LSM) {
		byte val = LSM.getPhase(x, y, z);
		if(val <= MTBLevelsetMembership.BG_PHASE) 
			return this.nu;
		else 
			return (-1.0) * this.nu;
	}
	
	@Override
	public double E(MTBLevelsetMembership LSM) {
		return this.nu * LSM.getPixelInside();
	}
	
	@Override
	public void updateParams(int x, int y, int z, byte newPhase, MTBLevelsetMembership LSM) {
	}

	@Override
	public  String 	toString() {
        return name + ": nu = " + nu;
    }

	@Override
	public void print( MTBLevelsetMembership LSM, PrintStream out, String indent) {
		out.println( indent + name + ": " + E( LSM));
	}

}
