/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5365 $
 * $Date: 2012-04-04 16:00:22 +0200 (Mi, 04. Apr 2012) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.MiToBo.segmentation.snakes.energies.paramAdapt;



import de.unihalle.informatik.Alida.annotations.ALDParametrizedClass;
import de.unihalle.informatik.MiToBo.segmentation.snakes.optimize.*;

/**
 * Parameter adaptation for Kass et al. internal energy.
 * 
 * @author moeller
 */
@ALDParametrizedClass

public abstract class MTBSnakeE_KassInt_ParamAdapt {

	/**
	 * Update function for alpha values.
	 * @param o					Current snake optimizer.
	 * @param curAlphas	Current alpha values.
	 * @return New updated alpha values.
	 */
	public abstract double[] alphaUpdate(SnakeOptimizerSinglePDE o,
			double[] curAlphas);

	/**
	 * Update function for beta values.
	 * @param o					Current snake optimizer.
	 * @param curBetas	Current beta values.
	 * @return New updated beta values.
	 */
	public abstract double[] betaUpdate(SnakeOptimizerSinglePDE o,
			double[] curBetas);

	/**
	 * Get the maximum value for parameter alpha.
	 * @return Maximal possible alpha value.
	 */
	public abstract Double getMaxAlpha();

	/**
	 * Get the maximum value for parameter beta.
	 * @return Maximal possible beta value.
	 */
	public abstract Double getMaxBeta();

	/**
	 * Get the minimum value for parameter alpha.
	 * @return Minimal possible alpha value.
	 */
	public abstract Double getMinAlpha();

	/**
	 * Get the minimum value for parameter beta.
	 * @return Minimal possible beta value.
	 */
	public abstract Double getMinBeta();

	@Override
	public abstract String toString();
}
