/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5438 $
 * $Date: 2012-04-13 11:29:50 +0200 (Fr, 13. Apr 2012) $
 * $Author: posch $
 * 
 */

// TODO:  output of intermediate results for plugin
// TODO: optional: filename base fuer intermediate LS
// TODO: javadoc for all energy types
package mtb_segmentation;

import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.gui.GenericDialog;
import ij.io.OpenDialog;
import ij.plugin.filter.PlugInFilter;
import ij.process.ImageProcessor;

import java.util.Vector;



import de.unihalle.informatik.Alida.exceptions.ALDException;
import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.exceptions.ALDProcessingDAGException;
import de.unihalle.informatik.MiToBo.core.datatypes.images.MTBImage;
import de.unihalle.informatik.MiToBo.core.datatypes.images.MTBImageByte;
import de.unihalle.informatik.MiToBo.segmentation.levelset.nonPDE.ui.LevelSet_Segmentation_NonPDE.EnergyType;


/**
 * Plugin for levelset segmentation
 * using nonPDE optimization and optionally topology preserving.
 * For details see {@link de.unihalle.informatik.MiToBo.segmentation.levelset.nonPDE.ui.LevelSet_Segmentation_NonPDE}
 * @author Stefan Posch, Markus Glass
 *
 */


public class LevelSet_Segmentation_NonPDE implements PlugInFilter {

	/** The levelset solver
     */
	de.unihalle.informatik.MiToBo.segmentation.levelset.nonPDE.ui.LevelSet_Segmentation_NonPDE mainOp;

    /*
     * (non-Javadoc)
     * @see ij.plugin.filter.PlugInFilter#setup(java.lang.String, ij.ImagePlus)
     */
	@Override
    public int setup(String arg, ImagePlus imp) {
        if (imp == null) {
                          IJ.showMessage("No image available, open one first!");
                          return DOES_ALL;
        }

		try {
			mainOp = new de.unihalle.informatik.MiToBo.segmentation.levelset.nonPDE.ui.LevelSet_Segmentation_NonPDE();
		} catch (ALDException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            IJ.showMessage("Cannot intantiate operator LevelSet_Segmentation_NonPDE!");
            return DOES_ALL;
        }

        mainOp.setInImg( MTBImage.createMTBImage(imp));
	
        return DOES_8G + DOES_16 + DOES_32;
    }

    /*
     * (non-Javadoc)
     * @see ij.plugin.filter.PlugInFilter#run(ij.process.ImageProcessor)
     */
	@Override
    public void run(ImageProcessor ip) {

		try {
        	if ( parameterDialog() ) {
				try {
            		// run operation
            		mainOp.runOp( null);
	
            		// get output image that was attached to the parameter object
            		MTBImage resultImg = mainOp.getResultImage();
            		resultImg.setTitle(this.mainOp.getInImg().getTitle() + " LevelSet_Segmentation_NonPDE");
            		resultImg.show();
		
					if ( mainOp.getSpacing() > 0 ) {
						Vector<MTBImageByte> intermediateLS = mainOp.getIntermediateLS();
						ImageStack stack = new ImageStack(mainOp.getInImg().getSizeX(), mainOp.getInImg().getSizeY());
						for ( int i = 0 ; i < intermediateLS.size() ; i++ ) {
							stack.addSlice( null, intermediateLS.elementAt(i).getImagePlus().getProcessor());
						}
						ImagePlus result1 = new ImagePlus( "Intermediate", stack);
						result1.show();
					}
				} catch (ALDException e) {
					// TODO Auto-generated catch block
        			e.printStackTrace();
    			}
        	}
		} catch (ALDException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            IJ.showMessage("LevelSet_Segmentation_NonPDE: error in dialog");
    	}
    }

    // ---- Plugin helper functions ----

	/**
	 * parameter dialog that is displayed when the plugin is started
	 * 
	 * @return
	 * @throws ALDOperatorException 
	 * @throws ALDProcessingDAGException 
	 */
	private boolean parameterDialog() throws ALDOperatorException, ALDProcessingDAGException
	{
		String[] energies = {"Chan-Vese energy", "Meijering fitting energy","Mean Separation"};
		String[] initMethods = {"binary mask", "threshold", "ellipse", "label image"};
		
		GenericDialog gd = new GenericDialog("settings");
		gd.addChoice("select initialization method", initMethods, initMethods[0]);
		gd.addChoice("Select energy", energies, energies[0]);
		gd.addNumericField("maximum number of iterations", 500, 0);
		gd.addCheckbox("multiphase", false);
		gd.addCheckbox("preserve topology", false);
		gd.addCheckbox("verbose", false);
		gd.addNumericField("spacing of intermediate results", 0, 0);
		
		gd.showDialog();
		
		if(gd.wasOKed())
		{
			String im = gd.getNextChoice();
			String en = gd.getNextChoice();

			mainOp.setMaxIter( (int)gd.getNextNumber());
			mainOp.setMultiphase( gd.getNextBoolean());
			mainOp.setPreserveTopology( gd.getNextBoolean());
			mainOp.setSpacing( (int)gd.getNextNumber());

			mainOp.setVerbose( gd.getNextBoolean());
			
			// --------------------------- initialize level set function ----------------------------------------
			if(im.equalsIgnoreCase(initMethods[0]))	{ // binary mask
				OpenDialog od = new OpenDialog("select mask file", "");
				String path = od.getDirectory() + od.getFileName();
				
				mainOp.setInitBinImg( MTBImage.createMTBImage(new ImagePlus(path)));
				
			} else if(im.equalsIgnoreCase(initMethods[1]))	{ // thresholding
				
				mainOp.setThreshold( IJ.getNumber("select threshold", 128));
				
			} else if(im.equalsIgnoreCase(initMethods[2]))	{ // ellipse
				
			} else	{ // label image
				OpenDialog od = new OpenDialog("select label file", "");
				String path = od.getDirectory() + od.getFileName();
				
				mainOp.setInitLabelImg( MTBImage.createMTBImage(new ImagePlus(path)));
			}
			
			// ---------------------------------------- choose energy -------------------------------------
			if(en.equalsIgnoreCase(energies[0]))	// Chan-Vese-Energy
			{
				mainOp.setEnergytype( EnergyType.CHANVESE);

				GenericDialog cvDialog = new GenericDialog("Chan-Vese energy settings");
				cvDialog.addNumericField("\u03BC", 0, 1);
				cvDialog.addNumericField("\u03BD", 0, 1);
				cvDialog.addNumericField("\u03BB FG", 1, 1);
				cvDialog.addNumericField("\u03BB BG", 1, 1);
				
				cvDialog.showDialog();
				
				if(cvDialog.wasOKed()) {
					mainOp.setMu( cvDialog.getNextNumber());
					mainOp.setNu( cvDialog.getNextNumber());
					mainOp.setLambdafg( cvDialog.getNextNumber());
					mainOp.setLambdabg( cvDialog.getNextNumber());
					
				} else {
				}

			}
			else if(en.equalsIgnoreCase(energies[1]))	{// Meijering fitting energy
				mainOp.setEnergytype( EnergyType.MEIJERING);

			} else if(en.equalsIgnoreCase(energies[2])) {
				mainOp.setEnergytype( EnergyType.MEANSEPARATION);

			}
			
			return true;
		}
		
		return false;
	}
}
